/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package Applet;

import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener; 
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ListIterator;

import javax.swing.BorderFactory;
import javax.swing.JLayeredPane;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.border.Border;
import dialogs.PaneBoutonsListeValeursEtFonctionsTextField;
import objets.CCalcul;
import objets.CCalculAncetre;
import objets.CCalculBase;
import objets.CEditeurFormule;
import objets.CElementBase;
import objets.CFonc;
import objets.CFoncNVar;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import types.Complexe;
import types.ErreurCalculException;
import types.FonteNom;
import types.Int;
import types.NatureCalcul;
import types.NatureObjet;
import types.OperateurFonction;
import types.OperateurFonction2Var;
import types.OperateurFonction3Var;
import types.OperateurFonction4Var;
import types.OperateurFonction5Var;
import types.StyleEncadrement;

@SuppressWarnings("serial")
public class MtgFormulaEditor extends JTextField implements KeyListener, FocusListener {
	private CEditeurFormule editeurFormuleAssocie;
 	private PaneBoutonsListeValeursEtFonctionsTextField paneBoutons = null;
 	private boolean erreur;	// Sera mis  true si une erreur dans le champ d'entre a t dtecte
 									// Dans ca cas, le champ est entour d'un cadre rouge.
 	private Border border;
 	private Int indErr = new Int(0);
 	
	public MtgFormulaEditor(CEditeurFormule editeurFormuleAssocieInit) {
		super(editeurFormuleAssocieInit.nbcol);
		setFont(FonteNom.getFonteNom(editeurFormuleAssocieInit.indiceTaillePolice, FonteNom.Normal));
		editeurFormuleAssocie = editeurFormuleAssocieInit;
		addKeyListener(this);
		addFocusListener(this);
		border = getBorder();
		// setFocusTraversalKeysEnabled(false); // Supprim version 4.6
	}
	
	public void keyPressed(KeyEvent evt) {
		CListeObjets liste = editeurFormuleAssocie.listeProprietaire;
		AppletMathGraph32 cadre = liste.documentProprietaire.applet;
		if ((evt.getKeyCode() == KeyEvent.VK_ENTER)) { // Appui sur la touche entre quand l'diteur a le focus
			retirePaneBouton();	
			if (validation()) {
				editeurFormuleAssocie.creeFormule(); // Ajout version 4.7.2
				editeurFormuleAssocie.premierAffichage = false;
				cadre.repaintFigureAvecCalcul();
				transferFocusToNextEditor(); // Validation avec message)
				demarquePourErreur();
			}
			else {
				setCaretPosition(indErr.getValue());				
				marquePourErreur();
			}
		}
		if ((evt.getKeyCode() == KeyEvent.VK_ESCAPE)) { // Appui sur la touche chappement quand l'diteur a le focus
			if (!validation()) marquePourErreur();
			else demarquePourErreur();
			retirePaneBouton();
			cadre.requestFocus();
		}
		// cadre.requestFocus();
	}
	
	void marquePourErreur() {
		if (!erreur) {
			setBorder(BorderFactory.createLineBorder(Color.red, 2));
			setToolTipText(AppletMathGraph32.getString("ch71"));
			erreur = true;
		}
	}
	
	public void demarquePourErreur() {
		if (erreur) {
			setBorder(border);
			setToolTipText("");
			erreur = false;
		}
	}

	public void keyReleased(KeyEvent evt) {
		// Modifi version 4.7.2.1 pour que l'affichage LaTeX, s'il est demand, s'affiche en temps rel
		if (editeurFormuleAssocie.affichageTempsReel && (evt.getKeyCode() != KeyEvent.VK_ENTER)) {
			String chCalcul = traiteSignesMult();
			editeurFormuleAssocie.premierAffichage = false;
			CListeObjets liste = editeurFormuleAssocie.listeProprietaire;
			AppletMathGraph32 cadre = liste.documentProprietaire.applet;
			CListeObjets listePr = cadre.listePr;
	    CCalcul calculAssocie = editeurFormuleAssocie.calculAssocie;
	    // On autorise d'utiliser un calcul non cr par l'utilisateur
	    int indDernier = listePr.indexOf(calculAssocie) - 1;
	    String[] varFor = null;
	    boolean bcomplexe = calculAssocie.estDeNatureCalcul(NatureCalcul.NCalculComplexe | NatureCalcul.NFonctionComplexe
	    		| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var);
	    if (calculAssocie.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
	    	CFonc f = (CFonc) calculAssocie;
	    	varFor = new String[1];
	    	varFor[0] = f.chaineVariable;
	    }
	    else {
	      if (calculAssocie.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
	      		| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
		    	CFoncNVar fnv = (CFoncNVar) calculAssocie;
		    	varFor = new String[fnv.nbVar];
		    	for (int k = 0; k < fnv.nbVar; k++) varFor[k] = new String(fnv.chaineVariable[k]);
	      }
	    }
	    if (bcomplexe ? CCalculBase.verifieSyntaxeComplexe(listePr, chCalcul, indErr, 
	        indDernier, varFor) : CCalculBase.verifieSyntaxe(listePr, chCalcul, indErr, 
	            indDernier, varFor)) {
	  		demarquePourErreur();
	    }
	    else {
	    	editeurFormuleAssocie.formula = null;
	    }
      CCalculBase calc = bcomplexe ? CCalculBase.creeCalculBaseComplexe(chCalcul ,listePr, 0,
          chCalcul.length()-1, varFor) : CCalculBase.creeCalculBase(chCalcul ,listePr, 0,
          chCalcul.length()-1, varFor);
      // try rajout version 4.7.3
      try {
      	editeurFormuleAssocie.creeFormule(calc.chaineLatexSansPar(varFor));
      }
      catch (Exception e) {
			}
			cadre.repaintFigureSansCalcul();
		}
	}

	public void keyTyped(KeyEvent evt) {
		// TODO Auto-generated method stub
		// Ajout version 4.9.6 : Si l'expression est correcte au niveau des parenthses et si le curseur n'est suivi
    // que d'espaces ou de parenthses fermantes, on rajoute une parenthse fermante
		if (evt.getKeyChar() == '(') {
	    int ind = getCaretPosition();
	    String ch = getText();
	    int s = 0;
	    for (int i = 0; i < ch.length(); i++) {
	      if (ch.charAt(i) == '(') s++;
	      else if (ch.charAt(i) == ')') s--;
	    }
	    boolean b = true;
	    for (int i = ind; i<ch.length(); i++) {
	      if ((ch.charAt(i) != ')') || (ch.charAt(i) != ')')) b = false;
	    }
	    if ((s==0) && b) {
	      setText(ch.substring(0,ind)+ "()" + ch.substring(ind));
	      setCaretPosition(ind+1);
	      evt.consume();
	    }
		}		
	}
	
	public void focusGained(FocusEvent arg0) {
		demarquePourErreur();
		CListeObjets listePr = editeurFormuleAssocie.listeProprietaire;
		AppletMathGraph32 cadre = listePr.documentProprietaire.applet;
		CListeObjets liste = cadre.listePourConstruction();
		int indmax;
		if (cadre.isExercise || cadre.isConstruction()) indmax = liste.size() - 1;
		else indmax = listePr.indexOf(editeurFormuleAssocie.calculAssocie) - 1;
		CMathGrafDoc doc = editeurFormuleAssocie.calculAssocie.listeProprietaire.documentProprietaire;
		boolean buttonValueAllowed = doc.buttonValueAllowed;
		boolean buttonFunctionAllowed = doc.buttonFunctionAllowed;
		if ((buttonValueAllowed || buttonFunctionAllowed) && (paneBoutons == null)) {
			paneBoutons = new PaneBoutonsListeValeursEtFonctionsTextField
					(cadre, this, editeurFormuleAssocie.calculAssocie.estDeNatureCalcul
							(NatureCalcul.NTouteFonctionReelle | NatureCalcul.NCalculReel),
							indmax, false, buttonValueAllowed, buttonFunctionAllowed);
			paneBoutons.setBackground(cadre.getBackground());
			paneBoutons.btnListeValeurs.setFocusable(false);
			paneBoutons.btnListeFonctionsPredefinies.setFocusable(false);
			cadre.springl.putConstraint(SpringLayout.NORTH, paneBoutons, 0, SpringLayout.SOUTH, this);
	    cadre.springl.putConstraint(SpringLayout.WEST, paneBoutons, 0, SpringLayout.WEST, this);
	    cadre.paneCenter.add(paneBoutons, JLayeredPane.DRAG_LAYER);
	    cadre.validate();
		}
		/*
		Component comp = arg0.getOppositeComponent();
		if (comp != null) {
			if( (comp.getClass() == MtgFormulaEditor.class) && (comp != this)) {
				focusLost = false;
			}
		}
		*/
	}

	public void focusLost(FocusEvent arg0) {
		CListeObjets liste = editeurFormuleAssocie.listeProprietaire;
		AppletMathGraph32 cadre = liste.documentProprietaire.applet;
		retirePaneBouton();
		if (validation()) {
			demarquePourErreur();
			cadre.repaintFigureAvecCalcul();
			return;  // Validation sans message
		}
		else {
			marquePourErreur();
			erreur = true;
		}
		/*
		Component comp = arg0.getOppositeComponent();
		// Si on a clique sur un bouton d'outil ou droul un menu on met le texte vide
		if (comp != null) {
			if ((comp.getClass() == MtgButton.class) || (comp.getClass() == JRootPane.class)) setText("");
		}
		if (arg0.isTemporary() || focusLost) return;
		validation(true); // Validation avec message
		focusLost = true;
		cadre.repaintFigureSansCalcul();
		*/
	}
	
	public final boolean validation() {
		if (getText().isEmpty()) return true;
		String chCalcul = traiteSignesMult();
    // Spcial version Applet !
		AppletMathGraph32 cadre = editeurFormuleAssocie.listeProprietaire.documentProprietaire.applet;
		CListeObjets liste = cadre.listePourConstruction();
		CListeObjets listePr = cadre.listePr;
    CCalcul calculAssocie = editeurFormuleAssocie.calculAssocie;
    // On autorise d'utiliser un calcul non cr par l'utilisateur
    int indDernier;
    // Modifi version 4.7.2
    // if (cadre.isExercise || cadre.isConstruction()) indDernier = liste.size() - 1;
    if (cadre.isExercise) indDernier = liste.size() - 1;
    else indDernier = listePr.indexOf(calculAssocie) - 1;
    String[] varFor = null;
    boolean bcomplexe = calculAssocie.estDeNatureCalcul(NatureCalcul.NCalculComplexe | NatureCalcul.NFonctionComplexe
    		| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var);
    if (calculAssocie.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
    	CFonc f = (CFonc) calculAssocie;
    	varFor = new String[1];
    	varFor[0] = f.chaineVariable;
    }
    else {
      if (calculAssocie.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
      		| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
	    	CFoncNVar fnv = (CFoncNVar) calculAssocie;
	    	varFor = new String[fnv.nbVar];
	    	for (int k = 0; k < fnv.nbVar; k++) varFor[k] = new String(fnv.chaineVariable[k]);
      }
    }
    if (bcomplexe ? CCalculBase.verifieSyntaxeComplexe(liste, chCalcul, indErr, 
        indDernier, varFor) : CCalculBase.verifieSyntaxe(liste, chCalcul, indErr, 
            indDernier, varFor)) {
      CCalculBase calc = bcomplexe ? CCalculBase.creeCalculBaseComplexe(chCalcul ,listePr, 0,
              chCalcul.length()-1, varFor) : CCalculBase.creeCalculBase(chCalcul ,listePr, 0,
              chCalcul.length()-1, varFor);
      if (calculAssocie.estDeNatureCalcul(NatureCalcul.NCalculReel | NatureCalcul.NCalculComplexe)) {
	      try {
	        calc.listeProprietaire.initialiseNombreIterations();
	      	if (bcomplexe) {
	          Complexe v = new Complexe();
	  				calc.resultatComplexe(false, v);
	      	}
	      	else {
		        @SuppressWarnings("unused")
						double v = calc.resultat(false);
	      	}
	      }
	      catch (ErreurCalculException e) {
	      	indErr.setValue(getText().length());
	      	return false;
	      }
      }
      calculAssocie.calcul = calc;
      // Modification version 4.5.2 : Quand la chane de calcul sera reconstruite elle pourra tre diffrente de celle ici entre
      // On la change tout de suite
      calculAssocie.chaineCalcul = calculAssocie.calcul.chaineCalculSansPar(varFor);
      //
      // Modification bug version 4.7.1 : On remet  jour les objets dpendant, en particulier les calculs de drives
      // Modifi version 4.7.2
		  listePr.metAJourObjetsDependantDeSaufEditeursFormule(calculAssocie);      		
      // On recalcule la figure mais on en la raffiche pas dans validation()
      cadre.positionne();
      cadre.gestionnaire.enregistreFigureEnCours(AppletMathGraph32.getString("chaineOutil48"));
      // cadre.repaintFigureAvecCalcul();  // Modifi version 4.6
      // setText("");  // Modifi version 4.6
      // cadre.requestFocus(); Modifi version 4.6
      return true;
    }
    else {
    	adapteIndiceErreur();
    	return false;
    }
	}

	// Ajout version 4.6
	/**
	 * Si l'diteur de formule associ a son paramtre variables1Car qui est  true, on rajoute les signes de multiplication implicites
	 *  la chane renvoye par getText()
	 * @return
	 */
	/* Modifi version 4.7.4.1 pour adapter aussi les noms de fonctions prdfinies
	private String traiteSignesMult() {
		String ch = getText();
		if ((!editeurFormuleAssocie.variables1Car) || ch.isEmpty()) return ch;
		StringBuffer stb = new StringBuffer(2*ch.length());
		for (int i = 0; i < ch.length() - 1; i ++) {
			char car = ch.charAt(i);
			char carsuiv = ch.charAt(i + 1);
			stb.append(car);
			// Si lettre suivie d'une autre lettre
			if (FonteNom.lettre(car) && FonteNom.lettre(carsuiv)) stb.append('*');
			// Si chiffre suivi d'une lettre
			if (FonteNom.chiffre(car) && FonteNom.lettre(carsuiv)) stb.append('*');		
			// Si chiffre ou lettre suivi d'une parenthse on rajoute un signe de multiplication
			if ((FonteNom.chiffre(car) || FonteNom.lettre(car)) && (carsuiv == '(')) stb.append('*');
			// Si parenthse fermante suivie d'un chiffre ou une lettre ou une parenthse ouvrante on rajoute un signe de multiplication
			if ((car == ')') && (FonteNom.lettre(carsuiv) || FonteNom.chiffre(carsuiv) || (carsuiv == '('))) stb.append('*');
		}
		stb.append(ch.charAt(ch.length() - 1));
		return stb.toString();
	}
	*/
	private String traiteSignesMult() {
		String ch = getText();
		Int lon = new Int(0);
		if ((!editeurFormuleAssocie.variables1Car) || ch.isEmpty()) return ch;
		StringBuffer stb = new StringBuffer(2*ch.length());
		char carprecedent = '(';
		for (int i = 0; i < ch.length(); i ++) {
			char car = ch.charAt(i);
			if (chiffreOuPointDecimal(car) || (car == ')') || (car == '+') || (car == '-') 
					|| (car == '*') || (car == '/') || (car == '^') || (car == '\'') || (car == '\"') || (car == '')) {
				carprecedent = car;
				stb.append(car);
			}
			else {
				if (ch.substring(i).startsWith("pi")) {
					if (chiffreOuPointDecimal(carprecedent) || lettreOuPrime(carprecedent) || (carprecedent == ')')) stb.append("*pi");
					else stb.append("pi");
					carprecedent = ')';
					i++;
				}
				else {
					if (commenceParNomFonctionPredefinieSuivieParOuvr(ch, i, lon)) {
						if (chiffreOuPointDecimal(carprecedent) || lettreOuPrime(carprecedent) || (carprecedent == ')')) stb.append("*"+ch.substring(i, i+lon.getValue()+1));
						else stb.append(ch.substring(i, i+lon.getValue()+1));
						i+= lon.getValue();
						carprecedent = '(';
					}
					else {
						if (commenceParNomFonctionUtilisateurSuivieParOurvr(ch, i, lon)) {
							if (chiffreOuPointDecimal(carprecedent) || lettreOuPrime(carprecedent) || (carprecedent == ')')) stb.append("*"+ch.substring(i, i+lon.getValue()+1));
							else stb.append(ch.substring(i, i+lon.getValue()+1));
							i+= lon.getValue();
							carprecedent = '(';							
						}
						else {
							// Si lettre suivie d'une autre lettre
							if (lettreOuPrime(carprecedent) && FonteNom.lettre(car)) stb.append('*');
							// Si chiffre suivi d'une lettre
							else if (chiffreOuCarre(carprecedent) && FonteNom.lettre(car)) stb.append('*');		
							// Si chiffre ou lettre suivi d'une parenthse on rajoute un signe de multiplication
							else if ((chiffreOuCarre(carprecedent) || lettreOuPrime(carprecedent)) && (car == '(')) stb.append('*');
							// Si parenthse fermante suivie d'un chiffre ou une lettre ou une parenthse ouvrante on rajoute un signe de multiplication
							else if ((carprecedent == ')') && (FonteNom.lettre(car) || FonteNom.chiffre(car) || (car == '('))) stb.append('*');
							stb.append(car);
							carprecedent = car;
						}
					}
				}
			}
		}
		return stb.toString();
	}

	// Ajout version 4.7.4.1
	private static final boolean chiffreOuCarre(char car) {
		return ((car == '') || FonteNom.chiffre(car));		
	}

	// Ajout version 4.7.4.1
	private static final boolean chiffreOuPointDecimal(char car) {
		return ((car == '.') || FonteNom.chiffre(car));
	}

	// Ajout version 4.7.4.1
	private static final boolean lettreOuPrime(char car) {
		return (FonteNom.lettre(car) || (car == '\'') || (car == '\"'));
	}
	/* Modifi version 4.7.4.1
	private void adapteIndiceErreur() {
		if (editeurFormuleAssocie.variables1Car) {
			String ch = getText();
			int ind = indErr.getValue();
			for (int i = 0; (i <= indErr.getValue()) && (i < ch.length() - 1); i ++) {
				char car = ch.charAt(i);
				char carsuiv = ch.charAt(i + 1);
				// Si lettre suivie d'une autre lettre
				if (FonteNom.lettre(car) && FonteNom.lettre(carsuiv)) ind--;
				// Si chiffre suivi d'une lettre
				if (FonteNom.chiffre(car) && FonteNom.lettre(carsuiv)) ind--;		
				// Si chiffre ou lettre suivi d'une parenthse on rajoute un signe de multiplication
				if ((FonteNom.chiffre(car) || FonteNom.lettre(car)) && (carsuiv == '(')) ind--;
				// Si parenthse fermante suivie d'un chiffre ou une lettre ou une parenthse ouvrante on rajoute un signe de multiplication
				if ((car == ')') && (FonteNom.lettre(carsuiv) || FonteNom.chiffre(carsuiv) || (carsuiv == '('))) ind--;
			}
			indErr.setValue(ind);
		}
	}
	*/
	private void adapteIndiceErreur() {
		Int lon = new Int(0);
		if (editeurFormuleAssocie.variables1Car) {
			String ch = getText();
			char carprecedent = '(';
			int k = 0; // Reprsente l'indice dans la chane  laquelle ont t rajouts des ventuels signs de multiplication
			int i = 0; // Reprsente l'indice dans la chae qui a t entre avec des multiplications implicites.
			int len = ch.length();
			while ((k <= indErr.getValue()) && (i < len)) {
				char car = ch.charAt(i);
				if (chiffreOuPointDecimal(car) || (car == ')') || (car == '+') || (car == '-')
						|| (car == '*') || (car == '/') || (car == '^') || (car == '\'') || (car == '\"') || (car == '')) {
					carprecedent = car;
					i++;
					k++;
				}
				else {
					if (ch.substring(i).startsWith("pi")) {
						k += 2;
						if (!(chiffreOuPointDecimal(carprecedent) || lettreOuPrime(carprecedent) || (carprecedent == ')'))) k++;
						carprecedent = ')';
						i += 2;
					}
					else {
						if (commenceParNomFonctionPredefinieSuivieParOuvr(ch, i, lon)) {
							k += lon.getValue();
							if (chiffreOuPointDecimal(carprecedent) || lettreOuPrime(carprecedent) || (carprecedent == ')')) k++;
							i+= lon.getValue();
							carprecedent = '(';
						}
						else {
							if (commenceParNomFonctionUtilisateurSuivieParOurvr(ch, i, lon)) {
								k += lon.getValue();
								if (chiffreOuPointDecimal(carprecedent) || lettreOuPrime(carprecedent) || (carprecedent == ')')) k++;
								i+= lon.getValue();
								carprecedent = '(';								
							}
							else {
								k++;
								i++;
								// Si lettre suivie d'une autre lettre
								if (lettreOuPrime(carprecedent) && FonteNom.lettre(car)) k++;
								// Si chiffre suivi d'une lettre
								else if (chiffreOuCarre(carprecedent) && FonteNom.lettre(car)) k++;		
								// Si chiffre ou lettre suivi d'une parenthse on rajoute un signe de multiplication
								else if ((chiffreOuCarre(carprecedent) || lettreOuPrime(carprecedent)) && (car == '(')) k++;
								// Si parenthse fermante suivie d'un chiffre ou une lettre ou une parenthse ouvrante on rajoute un signe de multiplication
								else if ((carprecedent == ')') && (FonteNom.lettre(car) || FonteNom.chiffre(car) || (car == '('))) k++;
								carprecedent = car;
							}
						}
					}
				}
			}
			indErr.setValue(i);
		}
	}
	
	// Ajout version 4.7.4.1
	/** Fonction renvoyant true si la sous-chane de pChaine commenant  l'indice indstart commence par un nom de fonction prdfinie relle ou complexe 
	 * 
	 * @param pChaine
	 * @param indstart
	 * @param longNom
	 * @return
	 */
	public static final boolean commenceParNomFonctionPredefinieSuivieParOuvr(
			String pChaine, int indstart, Int longNom) {
		String ch = pChaine.substring(indstart).toLowerCase();
		int i;
		// On commence par les fonctions de quatre variables car integrale commence par int une autre fonction
		for (i = 0; i < OperateurFonction4Var.nomsFonctions4Var.length; i++) {
			longNom.setValue(OperateurFonction4Var.nomsFonctions4Var[i].length());
			if (ch.startsWith(OperateurFonction4Var.nomsFonctions4Var[i].toLowerCase()+"(")) return true;
		}
		for (i = 0; i < OperateurFonction.nomsFonctions.length; i++) {
			longNom.setValue(OperateurFonction.nomsFonctions[i].length());
			if (ch.startsWith(
					OperateurFonction.nomsFonctions[i].toLowerCase()+"(")) return true;
		}
		for (i = 0; i < OperateurFonction2Var.nomsFonctions2Var.length; i++) {
			longNom.setValue(OperateurFonction2Var.nomsFonctions2Var[i].length());
			if (ch.startsWith(OperateurFonction2Var.nomsFonctions2Var[i].toLowerCase()+"(")) return true;
		}
		for (i = 0; i < OperateurFonction3Var.nomsFonctions3Var.length; i++) {
			longNom.setValue(OperateurFonction3Var.nomsFonctions3Var[i].length());
			if (ch.startsWith(OperateurFonction3Var.nomsFonctions3Var[i].toLowerCase()+"(")) return true;
		}
		for (i = 0; i < OperateurFonction5Var.nomsFonctions5Var.length; i++) {
			longNom.setValue(OperateurFonction5Var.nomsFonctions5Var[i].length());
			if (ch.startsWith(OperateurFonction5Var.nomsFonctions5Var[i].toLowerCase()+"(")) return true;
		}
		// Inutile de regarder les noms des fonctions complexes de plus de deux variables car c'est le mme que les fonctions relles
		for (i = 0; i < OperateurFonction.nomsFonctionsComplexes.length; i++) {
			longNom.setValue(OperateurFonction.nomsFonctionsComplexes[i].length());
			if (ch.startsWith(OperateurFonction.nomsFonctionsComplexes[i].toLowerCase()+"("))  return true;
		}
		return false;
	}
	
	public final boolean commenceParNomFonctionUtilisateurSuivieParOurvr(String pChaine, int indstart, Int longNom) {
		CListeObjets liste = editeurFormuleAssocie.listeProprietaire;
		CCalculAncetre ptf;
		String ch = pChaine.substring(indstart).toLowerCase();

		ListIterator<CElementBase> li = liste.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() 
					&& elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionReelle | NatureCalcul.NDerivee | NatureCalcul.NSuiteRecurrenteReelle
							| NatureCalcul.NTouteFonctionComplexe | NatureCalcul.NSuiteRecurrenteComplexe)) {
				ptf = (CCalculAncetre) elb;
				String nom = ptf.nom();
				if (ch.startsWith(nom + "(")) {
					longNom.setValue(nom.length());
					return true;
				}
			}
		}
		return false;
		
	}

	public final void setPositionOnInternaleFrame() {
		CListeObjets listePr = editeurFormuleAssocie.listeProprietaire;
		AppletMathGraph32 cadre = listePr.documentProprietaire.applet;
 		cadre.paneCenter.remove(this);
    cadre.paneCenter.add(this, new Integer(399));
    int decalagevert = (getPreferredSize().height - editeurFormuleAssocie.rectangleAffichage.height)/2;
    int decalagehor = (editeurFormuleAssocie.encadrement == StyleEncadrement.Sans) ? 0 : 3;
    int x = (int) (editeurFormuleAssocie.x_nom + editeurFormuleAssocie.dec_x + decalagehor);
    int y = (int) (editeurFormuleAssocie.y_nom + editeurFormuleAssocie.dec_y - decalagevert);
		cadre.springl.putConstraint(SpringLayout.WEST, this, x, SpringLayout.WEST, cadre);
		cadre.springl.putConstraint(SpringLayout.NORTH, this, y, SpringLayout.NORTH, cadre);
		// Il peut arriver qu'on implmente une construction dont l'objet source est un calcul non ditable.
		// Dans ce cas on ne permet pas la modification du champ d'dition
		if (!editeurFormuleAssocie.calculAssocie.estDeNatureCalcul(NatureCalcul.NCalculReelOuComplexe | NatureCalcul.NTouteFonctionReelleOuComplexe)) {
			setEditable(false);
			setFocusable(false);
		}
		cadre.validate();
	}
	
	public final void retirePaneBouton() {
		if (paneBoutons != null) 	{
			CListeObjets listePr = editeurFormuleAssocie.listeProprietaire;
			AppletMathGraph32 cadre = listePr.documentProprietaire.applet;
			cadre.remove(paneBoutons);
			paneBoutons = null;
			cadre.repaintFigureSansCalcul();
		}		
	}

	/**
	 * Ajout version 4.6.
	 * Pour transfrer le focus au prochain champ d'dition quand on a appuy sur entre et que la rponse est valide
	 */
	private void transferFocusToNextEditor() {
		CListeObjets listePr = editeurFormuleAssocie.listeProprietaire;
		int ind = listePr.indexOf(editeurFormuleAssocie) + 1;
		for (int i = 0; i < listePr.size() - 1; i++) {
			CElementBase elb = listePr.get((i + ind) % (listePr.size()));
			if (elb.getNature() == NatureObjet.NEditeurFormule) {
				CEditeurFormule ed = (CEditeurFormule) elb;
				if (ed.mtgFormulaEditor != this) {
					ed.mtgFormulaEditor.requestFocus();
					break;
				}
			}
		}
	}
}

