/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package Applet;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.IOException;
import java.util.ListIterator;

import javax.swing.JPanel;
import objets.CCommentaire;
import objets.CElementBase;
import objets.CListeObjets;
import objets.CMacro;
import objets.CMacroSuiteMacros;
import objets.CMathGrafDoc;
import outils.OutilPointage;
import outils.OutilPointageCapt;

/**
 *
 * @author Yves
 */
@SuppressWarnings("serial")
public class MtgPaneFigure extends JPanel implements MouseMotionListener, MouseListener {
  private AppletMathGraph32 appletMere;
  // Pour le double-buffering
  private Image imageBuffer;
  public Graphics2D gbuffer;
  // Nouveau pour la version 1.8 : un DeviceContext pour les macros laissant une trace
  private Image imageTraces;
  public Graphics2D tbuffer;

  public MtgPaneFigure(AppletMathGraph32 appletMereInit) {
    super();
    addMouseMotionListener(this);
    addMouseListener(this);
    appletMere = appletMereInit;
  }
  public void effacegBuffer() {
    // effaceFenetre(gbuffer);
		Image imageFond = appletMere.imageFond;
		if (appletMere.modeTrace) {
			transferetbufferversgbuffer();
		}
		else {
			effaceFenetre(gbuffer);
			if ((imageFond != null) && appletMere.document.imageFondVisible) {
				gbuffer.drawImage(imageFond, 0, 0, this);
			}
		}
  	
  }
  public void effacetBuffer() {
    effaceFenetre(tbuffer);
  }
  // Nouveau pour la version 1.8
  public synchronized void afficheToutDansgbuffer() {
  	CListeObjets liste = appletMere.listePr;
  	Color couleurFond = appletMere.couleurFond;
  	CMathGrafDoc document = appletMere.document;
    liste.afficheTout(gbuffer, appletMere.dimf, true, couleurFond, document.transparencePourSurfaces, appletMere.pref_antialiasing);
		if (appletMere.document.modeTraceActive) liste.affichePointsMarquesPourTrace(tbuffer, appletMere.dimf, 
				true, couleurFond, document.transparencePourSurfaces, appletMere.pref_antialiasing);
  }
  
	public synchronized void afficheToutDansgbufferSaufElementsDe(CListeObjets listeExclue) {
		Image imageFond = appletMere.imageFond;
		if ((imageFond != null) && appletMere.document.imageFondVisible) {
			gbuffer.drawImage(imageFond, 0, 0, this);
		}
		appletMere.listePr.afficheToutSaufElementsDe(gbuffer, appletMere.dimf, true,
				appletMere.couleurFond, appletMere.document.transparencePourSurfaces, listeExclue, appletMere.pref_antialiasing);
	}
	  
  public synchronized final void effaceFenetre(Graphics2D gc) {
    gc.setColor(appletMere.couleurFond); // N?cessaire car peut ?tre appel? en dehors de paint
    gc.fillRect(0,0, getWidth(), getHeight());
  }
  
  // Nouveau pour version 1.8 / Transfre l'image contenue dans le tbuffer (traces)
  // dans le gbuffer
  public void transferetbufferversgbuffer() {
    gbuffer.drawImage(imageTraces, 0, 0, this);
  }

  /**
   * Affichage dans le buffer lors de l'historique
   */
  public void afficheToutDansgbufferPourHist() {
    effaceFenetre(gbuffer);
    if ((appletMere.imageFond != null) && appletMere.document.imageFondVisible) {
       gbuffer.drawImage(appletMere.imageFond, 0, 0, this);
    }
    Color couleurFond = appletMere.couleurFond;
    float transparence = appletMere.document.transparencePourSurfaces;
    appletMere.listePr.afficheJusque(gbuffer, appletMere.dimf,appletMere.listeSauve.size()-1, false,
    		couleurFond, transparence, appletMere.pref_antialiasing);
    appletMere.listeConstruction.afficheJusque(gbuffer, appletMere.dimf,
            appletMere.historiqueDlg.indiceObjetEnCours, false, couleurFond, transparence, appletMere.pref_antialiasing);
  }
  
	/**
	 * Affichage dans le buffer lors de l'historique y compris les objets de la
	 * liste clignotante
	 */
	public void afficheToutDansgbufferPourHistSaufListeClig() {
		CMathGrafDoc doc = appletMere.document;
		appletMere.paneFigure.effacegBuffer();
		//  revoir
		Image imageFond = appletMere.document.imageFond;
		if ((imageFond != null) && doc.imageFondVisible) {
			appletMere.paneFigure.gbuffer.drawImage(imageFond, 0, 0, this);
		}
    appletMere.listePr.afficheNonMasquesJusqueSaufElementsDe(gbuffer, appletMere.dimf, appletMere.listeSauve.size()-1,
    		appletMere.listeClignotante, doc.couleurFond, doc.transparencePourSurfaces);
    
    appletMere.listeConstruction.afficheJusqueSaufElementsDe(gbuffer, appletMere.dimf, appletMere.historiqueDlg.indiceObjetEnCours,
    		appletMere.listeClignotante, false, doc.couleurFond, doc.transparencePourSurfaces, appletMere.pref_antialiasing);
	}
  
  
  // Nouveau pour version 1.8
  // Envoie lengthcontenu du gbuffer sur l'?cran sans passer par pain()
  public void dessinegbuffer(Graphics2D gc) {
    gc.drawImage(imageBuffer, 0, 0, this);
    
  }
	// Il faut redfinir la mthode update car par dfaut elle efface le contenu
	// de
	// la fentre avec la couleur de fond avant d'appeler paint(), ce qui provoque
	// un effet de flash
	@Override
	public void update(Graphics graphics) {
		paint(graphics);
	}
  
  @Override
  public void paint(Graphics graphics) {
    if (appletMere.chargementEnCours) return;
    Color couleurFond = appletMere.couleurFond;
    setBackground(couleurFond);
    setSize(appletMere.paneCenter.getSize()); // Ligne ncessaire car this n'a pas de taille. C'est paneCenter qui en une.
    // Ajout version 2.0
    OutilPointage outilPointage = appletMere.outilPointageActif;
		if (outilPointage != null) {
			if (outilPointage.aRedessinerDansPaint)	outilPointage.actionDansPaint(appletMere);
		}
    Graphics2D g2D = (Graphics2D) graphics;
    float transparence = appletMere.document.transparencePourSurfaces;
    CListeObjets listeObjetsVisuels = appletMere.listeObjetsVisuels;
    CListeObjets listePr = appletMere.listePr;
    CListeObjets listeSauve = appletMere.listeSauve;
    OutilPointage outilPointageActif = appletMere.outilPointageActif;
    OutilPointageCapt outilPointageCapture = appletMere.outilPointageCapture;
    CMacro macroAnimation = appletMere.macroAnimation;
    CMacro macroEnCours = appletMere.macroEnCours;
    CCommentaire commentaireDesignation = appletMere.commentaireDesignation;
    if (appletMere.premierAffichage) {
      setPreferredSize(getSize()); // Ligne ncessaire car this n'a pas de taille. C'est paneCenter qui en une.
      // Il faut donner  la liste des objets servant  la cration visuelle d'objets
      // la mme unit d'angle que celle de la liste principale
      listeObjetsVisuels.uniteAngle = listePr.uniteAngle;
      // Version 3.4.3 listePr.positionne(false, appletMere.dimf);
      // Version 3.4.3 listeSauve.setClone(listePr);
      // Si on vient de charger une nouvelle figure, inutile de recrer
      // les buffers;
      if (appletMere.adresseFigure == null) {   
        imageBuffer = createImage(getWidth(), getHeight());
        gbuffer = (Graphics2D) imageBuffer.getGraphics();
        imageTraces = createImage(getWidth(), getHeight());
        tbuffer = (Graphics2D) imageTraces.getGraphics();
      }
      // appletMere.initialisetbuffer(); // Modifi version 4.4.4
      appletMere.effacetBuffer(); // Ajoutr version 4.4.4
      effacegBuffer();
      
      // Bug corrig pour la version 2.5.4
      // Si l'applet est un exercice alatoire il faut recalculer alatoirement la figure
      // Modifi version 3.4.3
      // appletMere.infRand = true;
      // appletMere.positionne();
      // appletMere.infRand = false;
      // Ajout version 4.8
      CMacro macroDem = listePr.macroDemarrage;
      listePr.positionne(true, appletMere.dimf);
      if (macroDem != null) {
      	if (macroDem.getClass() == CMacroSuiteMacros.class){
      		CMacroSuiteMacros mac = (CMacroSuiteMacros) macroDem;
          ListIterator<CElementBase> li = mac.listeAssociee.listIterator();
          while (li.hasNext()) {
            CMacro mac2 = (CMacro) (li.next());
            mac2.actionAvantExecution(appletMere);
          }
      	}
      	else macroDem.actionAvantExecution(appletMere);      	
      }
      listeSauve.setClone(listePr);
  		// Modification version 4.6 : Lors du premier chargement vi un flux URL c'est ici qu'il faut initialiser le gestionnaire d'dition
  		try {
  			appletMere.gestionnaire.initialise();
  		}
  		catch (IOException e) {
  			// TODO: handle exception
  		}

      if (macroAnimation != null) {
        macroAnimation.actionAvantExecution(appletMere);
        if (appletMere.animationTimer == null) {
          appletMere.animationTimer = new java.util.Timer();
          appletMere.animationTimer.schedule(appletMere.timerTaskAnimation, 0l, macroAnimation.frequenceAnimation());
        }
      }
      appletMere.activeOutilCapture();
  		appletMere.isReady = true; // Ajout version 4.6.2. Permettra lors de l'appel de isReady de l'applet de savoir si l'applet a bien t affiche.
    }
    // Ajout version 3.4.3 : Si on demande de recalculer la figure cela doit se faire dans paint()
    if (appletMere.aRecalculerEtRedessiner) {
    	appletMere.positionne();
  		effacegBuffer();
  		afficheToutDansgbuffer();
  		appletMere.aRecalculerEtRedessiner = false;
  		appletMere.infRand = false;
    }
    else {
	  	if (appletMere.aRedessiner) {
	  		effacegBuffer();
	  		afficheToutDansgbuffer();
	  		appletMere.aRedessiner = false;
	  	}
    }
  
    // Fonctionnement en double buffering
    if (macroAnimation != null) {
      macroAnimation.actionDansPaint(appletMere);
    }
    if (outilPointageActif == outilPointageCapture) {
      // Si une macro est en cours d'ex?cution
      if (macroEnCours != null) {
        CMacro mac = macroEnCours.macroEnCours();   // Dans le cas o la maco en cours est une
                                                    // CMacroSuiteMacros, mac pointe sur la macro
                                                    // de la liste en cours d'ex?cution
        if (mac.executionEnCours) {
          mac.actionDansPaint(appletMere);
          if ((mac.executionEnCours == false) && (mac.typeAnimation() == CMacro.sansAnimation)) {
            // termineAtion est ? true sauf dans lengthcas d'une CMacroSuiteMacro quand la macro en cours
            // n'est pas la derni?re
            mac.termineAction(appletMere);
						if (!(appletMere.macroEnCours instanceof CMacroSuiteMacros))
							appletMere.macroEnCours = null;
            
            // Modification version 3.0
            // if (appletMere.macroEnCours.macroEnCours() == appletMere.macroEnCours) {
            //  if (!appletMere.macroEnCours.executionEnCours) appletMere.macroEnCours = null;
            //}
            
            //if (!(macroEnCours instanceof CMacroSuiteMacros)) macroEnCours = null;

          }
          else {
            if (mac.dureePourAnimation() != 0) {
              if (System.currentTimeMillis() >= mac.timeFin) {
                mac.termineAction(appletMere);
                mac.executionEnCours = false;
                if (!(macroEnCours instanceof CMacroSuiteMacros))  macroEnCours = null;
              }
            }
          }
        }
        if (macroEnCours != null) {
          if (macroEnCours instanceof CMacroSuiteMacros) {
            if (!macroEnCours.macroEnCours().executionEnCours) {
              ((CMacroSuiteMacros) macroEnCours).passageMacroSuiv(appletMere);
              // passageMacroSuiv redonne ventuellement un ordre paint() suivant la nature de la macro suivante
              // passageMacroSuiv met executionEnCours  false si la dernire macro a t excute
              if (!macroEnCours.executionEnCours) {
                macroEnCours.termineAction(appletMere);
                appletMere.macroEnCours = null;
              }
            }
          }
        }
        dessinegbuffer(g2D);
      }
      else
      {
        // Si un point n'est pas captur, affichage classique
        // if (appletMere.premierAffichage) afficheToutDansgbuffer();
        // afficheToutDansgbuffer();
        if (commentaireDesignation.chaineCommentaire.length() != 0) {
          commentaireDesignation.positionne(appletMere.infRand, appletMere.dimf);
          dessinegbuffer(g2D);
          commentaireDesignation.affiche(g2D, true, couleurFond, transparence);
        }
        else dessinegbuffer(g2D);
      } 
      appletMere.premierAffichage = false;   
    }
    else {
      // Si on est en cration visuelle d'objets,
      // if (appletMere.premierAffichage) afficheToutDansgbuffer();
      dessinegbuffer(g2D);
      listeObjetsVisuels.positionne(appletMere.infRand, appletMere.dimf);
      listeObjetsVisuels.afficheTout(g2D, appletMere.dimf, true, couleurFond, appletMere.document.transparencePourSurfaces,
      		appletMere.pref_antialiasing);
      if (commentaireDesignation.chaineCommentaire.length() != 0) {
        commentaireDesignation.positionne(appletMere.infRand, appletMere.dimf);
        commentaireDesignation.affiche(g2D, true, couleurFond, transparence);        
      }
    }
  }
	public void mouseDragged(MouseEvent e) {
    if (appletMere.chargementEnCours) return;
    appletMere.outilPointageActif.mouseDragged(e);
		Cursor c = appletMere.outilActif.curseurUtilise();
		if (getCursor() != c) setCursor(c);
		
	}
	
	public void mouseMoved(MouseEvent e) {
    if (appletMere.chargementEnCours) return;
    appletMere.pointSouris.placeEn(e.getX(), e.getY());
    appletMere.outilPointageActif.mouseMoved(e);
		Cursor c = appletMere.outilActif.curseurUtilise();
		if (getCursor() != c) setCursor(c);
	}
	
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	public void mouseEntered(MouseEvent e) {
  	appletMere.showStatus(AppletMathGraph32.messageCopyright);		
	}
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	public void mousePressed(MouseEvent e) {
    if (appletMere.chargementEnCours) return;
    appletMere.outilPointageActif.mousePressed(e);
	}
	
	public void mouseReleased(MouseEvent e) {
    if (appletMere.chargementEnCours) return;
    appletMere.outilPointageActif.mouseReleased(e);
		Cursor c = appletMere.outilActif.curseurUtilise();
		if (getCursor() != c) setCursor(c);     
	}
}
