/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;
import types.DimensionFenetre;
import types.NatureCalcul;

/**
 * Classe cre pour la version 4.5.0 servant  rendre l'abscisse  l'origine d'un repre.
 * Principalement cr pour pouvoir crer une macro-construction dont l'objet source est un repre
 * et pouvoir  l'intreur de cette construction connatre l'abscisse  l'origine du repre.
 * @author Yves
 *
 */
public class CAbscisseOrigineRep extends CCalculAncetre {
	public CRepere rep;
	protected double abscisseOrigine;
	
	/**
	 * Constructeur pour serialisation
	 */
  public CAbscisseOrigineRep() {
  }
  
	/**
	 * Constructeur
	 */
  public CAbscisseOrigineRep(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit, String nomCalculInit,
      CRepere repInit) {
    // Inutile d'appeler le constructeur anctre vide
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    rep = repInit;
  }
	
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb =  new CAbscisseOrigineRep(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CRepere)(listeCible.get(ind1, CRepere.class)));
    return ptelb;
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NAbscisseOrigineRep;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    abscisseOrigine = ((CAbscisseOrigineRep) ptel).abscisseOrigine;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  getString("chinfo178") + " " + nomCalcul + " " + getString("chinfo32") + " " + rep.nom();
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + Kernel.chaineNombre(abscisseOrigine, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;        
  }
    
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || rep.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || rep.dependDePourBoucle(p);
  }

  @Override
	public
  double rendValeur(boolean infoRandom) {
    return abscisseOrigine;
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	existe = rep.existe;
  	abscisseOrigine = rep.valAbscisseOrigine;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CAbscisseOrigineRep pt;
    if (p.getClass() == this.getClass()) {
      pt = (CAbscisseOrigineRep) p;
      return rep == pt.rep;
    }
    else return false;
  }
    
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(rep);
    oups.writeInt(ind1);
  } 
  
}
