/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import types.ErreurCalculException;

/**
 * Classe ajoute pour la version 3.0 et servant dans le calcul de drive
 * @author Yves
 *
 */
public class CAppelFonctionInterneNVar extends CCalculBase {
	public CCalculBase fonctionInterne;
	public CCalculBase operande[];

  /** Creates a new instance of CAppelFonctionInterne */
  public CAppelFonctionInterneNVar(CCalculBase fonctionInterneInit, CCalculBase[] operandeInit) {
    fonctionInterne = fonctionInterneInit;
    operande = operandeInit;
  }
  
  @Override
  protected long nature() {
  	return natAppelFonctionInterneNVar;
  }  
  
  public double resultat(boolean infoRandom) throws ErreurCalculException {
  	int nvar = operande.length;
  	double[] d = new double[nvar];
  	for (int i = 0; i < nvar; i++ ) d[i] = operande[i].resultat(infoRandom);
    return fonctionInterne.resultatFonction(infoRandom,d);
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double valeurparametre) throws ErreurCalculException {
  	int nvar = operande.length;
  	double[] d = new double[nvar];
  	for (int i = 0; i < nvar; i++ ) d[i] = operande[i].resultatFonction(infoRandom, valeurparametre);
    return fonctionInterne.resultatFonction(infoRandom,d);
  }

  @Override
  public double resultatFonction(boolean infoRandom, double valeurparametre[]) throws ErreurCalculException {
  	int nvar = operande.length;
  	double[] d = new double[nvar];
  	for (int i = 0; i < nvar; i++ ) d[i] = operande[i].resultatFonction(infoRandom, valeurparametre);
    return fonctionInterne.resultatFonction(infoRandom,d);
  }
  
  public CCalculBase getCopie() {
  	int nvar = operande.length;
  	CCalculBase[] tab = new CCalculBase[nvar];
  	for (int i = 0; i < nvar; i++) tab[i] = operande[i].getCopie();
    return new CAppelFonctionInterneNVar(fonctionInterne.getCopie(), operande);
  }
  
  public boolean dependDeVariable(int ind)
  {
  	boolean res = false;
  	int nvar = operande.length;
  	for (int i = 0; i < nvar; i++) {
    	// Modifi version 4.1
  		// res = res || operande[i].dependDeVariable(ind);
  		res = res || (fonctionInterne.dependDeVariable(i) && operande[i].dependDeVariable(ind));  		
  	}
		return res;  
	}
  
  // Il faut dfinir getClone car elle est abstract dans CCalcul
  // mais elle est inutile car ce type d'objet n'est pas enregistr dans un flux
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    return null;
  }
  
  @Override
	public boolean estConstant() {
  	boolean res = false;
  	int nvar = operande.length;
  	for (int i = 0; i < nvar; i++) {
  		res = res || operande[i].estConstant();
  	}
		return res;
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	boolean b = true;
  	for (int i = 0; (i < operande.length) && b; i++) 
  		if (operande[i].dependDeVariable(indiceVariable)) 
  			b = fonctionInterne.deriveePossible(i) && operande[i].deriveePossible(indiceVariable);
  	return b;
  }
  
  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	int nbvar = fonctionInterne.nombreVariables();
  	CCalculBase[] tabcal = new CCalculBase[fonctionInterne.nombreVariables()];
  	for (int i = 0; i < nbvar; i++) {
  		tabcal[i] = operande[i].calculAvecValeursRemplacees();
  	}
  	return new CAppelFonctionInterneNVar(fonctionInterne, tabcal);
  }

  @Override
	public CCalculBase membreGauche() {
  	return fonctionInterne.membreGauche();
	}
	
	// Ajout version 4.6
	public CCalculBase membreDroit() {
  	return fonctionInterne.membreDroit();
	}

}
