/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import java.awt.Color;
import java.io.IOException;
import types.DimensionFenetre;
import types.NatureObjet;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * Ajout version 4.8. Utile pour les constructions itratives et rcursives
 */

public class CDroiteClone extends CDroite {
	private CDroiteAncetre droite; // La droite clone

  /** Constructeur */
  public CDroiteClone() {
  }
  
  public CDroiteClone(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      String nomInit, int indiceTailleNomInit, int styleInit, double abscisseNomInit, CDroiteAncetre droiteInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
    droite = droiteInit;
  }

  /**
   * Constructeur plus court pour cration d'objet
   * @param liste
   * @param impProtoInit
   * @param estElementFinalInit
   * @param couleurInit
   * @param masqueInit
   * @param styleInit
   * @param droiteInit
   */
  public CDroiteClone(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleInit, CDroiteAncetre droiteInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    droite = droiteInit;
  }

  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(droite);
    int ind2 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CDroiteClone(listeCible,(CImplementationProto)(listeCible.get(ind2,CImplementationProto.class)),
    	estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, style,
    	abscisseNom, (CDroiteAncetre)(listeCible.get(ind1, CDroiteAncetre.class)));
    return ptelb;
  }

  @Override
  public long getNature() {
    return NatureObjet.NDroiteClone;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || droite.dependDe(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || droite.dependDePourBoucle(p));
  }

  @Override  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	// Il faut mmoriser si le'objet est masqu ou non car setClone va le rinitialiser  la valeur du point copi
  	boolean b = masque;
  	setClone(droite);
  	masque = b;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    droite = (CDroiteAncetre) (list.get(ind1,CPointAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(droite);
    oups.writeInt(ind1);
  }

}
