/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * Classe reprsentant une fonction d'une variable dfinie par l'utilisateur
 * @author Yves
 *
 */
public class CFonc extends CCalcul {
	public String chaineVariable; // Reprsente la variable formelle

  /** Creates new CFonc */
  public CFonc() {
  }
  
  /** Constructeur pour outil de cration */
  public CFonc(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  
  // Constructeur pour clonage
  public CFonc(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit,String chaineCalculInit,
      CCalculBase calculInit, String chaineVariableInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit, chaineCalculInit, calculInit);
    chaineVariable = new String(chaineVariableInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    CElementBase ptelb = new CFonc(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone, chaineVariable);
    return ptelb;    
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NFonction;
  }
  
  @Override
  public int nombreVariables() {
  	return 1;
  }
  
  public double rendValeurFonction(boolean infoRandom, double parametre)
  throws ErreurCalculException {
    return calcul.resultatFonction(infoRandom, parametre);
  }
  
  // Ajout version 4.9.7 car on peut maintenant utiliser dans la formule d'un calcul ou d'une fonction complexe une fonction relle.
  // Transfr depuis CFoncComplexe
  // La formule relle sera utilise alors dans C.
  void rendValeurFonctionComplexe(boolean infoRandom, Complexe parametre, Complexe zRes)
      throws ErreurCalculException {
    calcul.resultatFonctionComplexe(infoRandom, parametre, zRes);
  }


	@Override
	public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		// Modifi version 4.9.5.4
	  // existe = calcul.existe(infoRandom);
		existe = calcul.existePourFonc(infoRandom);
	}
	
  /***
   * Fonction reconstruisant la chaine reprsentant le calcul (par exemple lorsqu'un autre calcul a t renomm
   * A redfinir pour les fonctions
   */
  public void reconstruitChaineCalcul() {
  	String[] ch = new String[1];
  	ch[0] = chaineVariable;  	
  	chaineCalcul = calcul.chaineCalculSansPar(ch);	
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("ch86") + " " + nom() + " " + getString("ch87") + "\n" + nom() 
    	+ "(" + chaineVariable + ")" + " = " + chaineCalcul;
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }
  
  /**
   * Fonction renvoyant une chaine du type f(t) = .... pour certain champs de botes de dialogues
   * @return
   */
  public String infoFormule() {
  	return nom() + "(" + chaineVariable + ")" + " = " + chaineCalcul;
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   // Inutile de redfinir informationModification() pour les objets de type calcul
  @Override
  public boolean aPourParametre(String ch) {
  	return chaineVariable.equals(ch);
  	
  }  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    chaineVariable = inps.readCString();
  }

  // Version 1.9.5 : Suite  l'introduction de CDerivee
  // il a fallu dfinir cette produre car CDerivee n'enregistre
  // pas le calcul dans le flux.
   public void chargeDepuisFluxSansCalcul(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFluxSansCalcul(inps,list);
  }
 
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    chaineVariable = inps.readUTF();
  }

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(chaineVariable);
  }

  @Override
  public void readSansCalcul(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.readSansCalcul(inps, list);
  }  
  /* Enregistrement dans un flux java */
  public void writeSansCalcul(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.writeSansCalcul(oups, list);
  }

  @Override
  public boolean estFonctionOuSuite() {
    return true;
  }
  
}

