/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import java.io.IOException;
import types.Complexe;
import types.ErreurCalculException;
import types.OperateurFonction3Var;

public class CFonction3Var extends CCalculBase {
	
  // Attibut rajout pour optimisation
  Complexe z1loc = new Complexe();
  Complexe z2loc = new Complexe();
  Complexe z3loc = new Complexe();
	
  int operateurFonction;
  CCalculBase operande1, operande2, operande3;

  /** Creates new CFonction */
  public CFonction3Var() {
  }

  /** Constructeur pour clonage */
  public CFonction3Var(CListeObjets liste, int operateurFonctionInit, CCalculBase operande1Init, CCalculBase operande2Init,
  		CCalculBase operande3Init) {
    super(liste);
    operateurFonction = operateurFonctionInit;
    operande1 = operande1Init;
    operande2 = operande2Init;
    operande3 = operande3Init;
  }
  
  @Override
  public int nombreVariables() {
  	return 3;
  }
  
  @Override
  protected long nature() {
  	return natFonction3Var;
  }  

  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande1 = operande1.getClone(listeSource, listeCible); 
    CCalculBase cloneOperande2 = operande2.getClone(listeSource, listeCible); 
    CCalculBase cloneOperande3 = operande3.getClone(listeSource, listeCible); 
    CCalculBase ptcalc = new CFonction3Var(listeCible, operateurFonction, cloneOperande1, cloneOperande2,
    		cloneOperande3);
    return ptcalc;
  }
  
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	operande1.initialisePourDependance();
  	operande2.initialisePourDependance();
  	operande3.initialisePourDependance();
  }

  // Modifi version 4.8
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	// elementTestePourDependDe = p;
    return memoriseDependance(super.dependDe(p) || operande1.dependDe(p) || operande2.dependDe(p) || operande3.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return operande1.dependDePourReclassement(p) || operande2.dependDePourReclassement(p)
    	|| operande3.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return operande1.dependDePourBoucle(p) || operande2.dependDePourBoucle(p)
    	|| operande3.dependDePourBoucle(p);
  }
  
  @Override
  public boolean existe(boolean infoRandom) {
  	double x1;
  	// Modifi version 3.5.1 : Si operande1 existe et s'il renvoie la valeur 1 existe renverra true
  	// si et seulement operande2 existe et s'il renvoie une autre valeur existe renverra true
  	// ssi operande3 existe
    // return operande1.existe(infoRandom) && operande2.existe(infoRandom) && operande3.existe(infoRandom);
  	if (!operande1.existe(infoRandom)) return false;
  	/* Modifi version 4.8
  	// On calcule la valeur renvoye par l'oprande de test.
    try {
			x1 = operande1.resultat(infoRandom);
		} catch (ErreurCalculException e) {
			return false;
		}
		if (x1 == 1) return operande2.existe(infoRandom);
		else return operande3.existe(infoRandom);
		*/
  	/* Modif version 4.9.0.1 pour corriger un bug
  	if (operande1.getClass() == CResultatValeur.class) {
  		double d = ((CResultatValeur) operande1).resultat(infoRandom);
  		return (d == 1) ? operande2.existe(infoRandom) : operande3.existe(infoRandom);
  	}
  	else {
  		if (operande1.getClass() == CResultatValeurComplexe.class) {
  			Complexe z  = new Complexe();
  			((CResultatValeurComplexe)operande1).resultatComplexe(infoRandom, z);
  			if ((z.y == 0) && (z.x == 1)) return operande2.existe(infoRandom); else return operande3.existe(infoRandom);
  		}
  		else return operande2.existe(infoRandom) && operande3.existe(infoRandom);
  	}
  	*/
  	if (operande1.getClass() == CResultatValeur.class) {
  		double d = ((CResultatValeur) operande1).resultat(infoRandom);
  		return (d == 1) ? operande2.existe(infoRandom) : operande3.existe(infoRandom);
  	}
  	else {
  		if (operande1.getClass() == CResultatValeurComplexe.class) {
  			Complexe z  = new Complexe();
  			((CResultatValeurComplexe)operande1).resultatComplexe(infoRandom, z);
  			if ((z.y == 0) && (z.x == 1)) return operande2.existe(infoRandom); else return operande3.existe(infoRandom);
  		}
  		else {
  	    try {
  				x1 = operande1.resultat(infoRandom);
  			} catch (ErreurCalculException e) {
  				return false;
  			}
  			if (x1 == 1) return operande2.existe(infoRandom);
  			else return operande3.existe(infoRandom);
  		}
  	}
  }
	// Ajout version version 4.9.5.4.
	// Pour une fonction relle ou complexe de une ou plusieurs variables, quand on teste son existence, on appellait
	// existe. Or CFonction3Var.existe, dans le cas d'une fonctio si provoque un appel de calcul pour un calcul normal qu'il faut viter quand
	// une fonction utilise un si et qu'on teste son existence
	public boolean existePourFonc(boolean infoRandom) {
		return operande1.existePourFonc(infoRandom) && operande2.existePourFonc(infoRandom) && operande3.existePourFonc(infoRandom);
	}
      
  public double resultat(boolean infoRandom) throws ErreurCalculException {
    double x1 = operande1.resultat(infoRandom);
    if (x1 == 1) return operande2.resultat(infoRandom);
    else return operande3.resultat(infoRandom);
  }

  // A revoir version 3/0. Inutile ?
  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
    double x1 = operande1.resultatFonction(infoRandom, valeurParametre);
   if (x1 == 1) return operande2.resultatFonction(infoRandom, valeurParametre);
   else return operande3.resultatFonction(infoRandom, valeurParametre);
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
    double x1 = operande1.resultatFonction(infoRandom, valeurParametre);
    if (x1 == 1) return operande2.resultatFonction(infoRandom, valeurParametre);
    else return operande3.resultatFonction(infoRandom, valeurParametre);
  }
      
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
    operande1.resultatComplexe(infoRandom, z1loc);
    if (z1loc.y != 0) throw new ErreurCalculException();
    if (z1loc.x == 1) {
    	operande2.resultatComplexe(infoRandom, z2loc);
    	zRes.x = z2loc.x;
    	zRes.y = z2loc.y;
    }
    else {
    	operande3.resultatComplexe(infoRandom, z3loc);
    	zRes.x = z3loc.x;
    	zRes.y = z3loc.y;
    }
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
    throws ErreurCalculException {
    operande1.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
    if (z1loc.y != 0) throw new ErreurCalculException();
    if (z1loc.x == 1) {
    	operande2.resultatFonctionComplexe(infoRandom, valeurParametre, z2loc);
    	zRes.x = z2loc.x;
    	zRes.y = z2loc.y;
    }
    else {
    	operande3.resultatFonctionComplexe(infoRandom, valeurParametre, z3loc);
    	zRes.x = z3loc.x;
    	zRes.y = z3loc.y;
    }
  }
  
  // Version 3.5.1 : Cette fonction avait t oublie dans les versions prcdentes ce qui fait que les composes
  // de fonctions complexes de 3 variables ne marchainet pas bien
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre[], Complexe zRes)
  throws ErreurCalculException {
    operande1.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
    if (z1loc.y != 0) throw new ErreurCalculException();
    if (z1loc.x == 1) {
    	operande2.resultatFonctionComplexe(infoRandom, valeurParametre, z2loc);
    	zRes.x = z2loc.x;
    	zRes.y = z2loc.y;
    }
    else {
    	operande3.resultatFonctionComplexe(infoRandom, valeurParametre, z3loc);
    	zRes.x = z3loc.x;
    	zRes.y = z3loc.y;
    }
  }  
  
  public boolean  dependDeVariable(int ind) {
    return operande1.dependDeVariable(ind) || operande2.dependDeVariable(ind) || operande3.dependDeVariable(ind);
  }
  
  public CCalculBase getCopie() {
   	return new CFonction3Var(listeProprietaire, operateurFonction, operande1.getCopie(), operande2.getCopie(),
   			operande3.getCopie());
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String cha1, cha2, cha3;
  	cha1 = operande1.chaineCalculSansPar(varFor);
  	cha2 = operande2.chaineCalculSansPar(varFor);
  	cha3 = operande3.chaineCalculSansPar(varFor);
  	String ch = parouv + cha1 + virg + cha2 + virg + cha3 + parfer;

  	switch (operateurFonction)
  	{
  		case OperateurFonction3Var.si:
  		case OperateurFonction3Var.siC:
  			return getString("si") + ch;  			
  		default : return "";
  	}
  }
 
  @Override
	public String chaineLatex(String[] varFor) {
  	String cha1, cha2, cha3;
  	cha1 = operande1.chaineLatexSansPar(varFor);
  	cha2 = operande2.chaineLatexSansPar(varFor);
  	cha3 = operande3.chaineLatexSansPar(varFor);
  	String ch = parouvLatex + cha1 + virg + cha2 + virg + cha3 + parferLatex;

  	switch (operateurFonction)
  	{
  		case OperateurFonction3Var.si:
  		case OperateurFonction3Var.siC:
  			return getString("si") + ch;  			
  		default : return "";
  	}
  }
 
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return operande1.estDefiniParObjetDans(listeOb) && operande2.estDefiniParObjetDans(listeOb)
  		&& operande3.estDefiniParObjetDans(listeOb);
	}	   
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operateurFonction = inps.readByte();
    operande1 = (CCalculBase) inps.readObject(list);
    operande2 = (CCalculBase) inps.readObject(list);
    operande3 = (CCalculBase) inps.readObject(list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(operateurFonction);
    oups.writeObject(operande1);
    oups.writeObject(operande2);
    oups.writeObject(operande3);
  }
  
  @Override
	public boolean estConstant() {
		return (operande1.estConstant() && operande2.estConstant() && operande3.estConstant());
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return operande1.deriveePossible(indiceVariable)
  	&& operande2.deriveePossible(indiceVariable) && operande3.deriveePossible(indiceVariable);
  }

  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CFonction3Var(listeProprietaire, operateurFonction, operande1.calculAvecValeursRemplacees(),
  			operande2.calculAvecValeursRemplacees(), operande3.calculAvecValeursRemplacees());
  }
  
}
