/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
/*

 *
 * Created on 14 juillet 2004, 18:04
 */

/**
 *
 * @author  Yves
 * @version
 */
import java.io.*;
import java.awt.Color;


import types.DimensionFenetre;
import types.InfoLieu;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

/**
 * Lieu de points gnr par une variable
 * @author Yves
 *
 */
public class CLieuParVariable extends CLieuDeBase {
  CVariableBornee  variableGeneratrice;
  
  /** Creates new CLieuParVariable */
  public CLieuParVariable() {
  }
  
  // Constructeur pour clonage
  public CLieuParVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit,
    CPointAncetre pointATracerInit, InfoLieu infoLieuInit,
    CVariableBornee variableGeneratriceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, pointATracerInit, infoLieuInit);
    variableGeneratrice = variableGeneratriceInit;
    // Rajout version 3.4.2
    // etablitListeElementsAncetres(); Inutile et provoquait un bug sous version 4.4 en appelant getClone
  }
  
  /**
   * A partir de la version 3.4.2, changement de version (version 1 -> 2) car on enregistre dans le flux la CSousListeObjetstres
   * contenant les pointeurs sur les lments anctres
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    InfoLieu infoLieuClone = new InfoLieu(infoLieu);
    int ind1 = listeSource.indexOf(pointATracer);
    int ind2 = listeSource.indexOf(variableGeneratrice);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CLieuParVariable(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      infoLieuClone, (CVariableBornee)(listeCible.get(ind2, CVariableBornee.class)));
    if (listeCible.getClass() != CPrototype.class) {
	    ((CLieuParVariable)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire,
	      ptelb.listeProprietaire);
    }
    return ptelb;
  }
  
  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   */
  @Override
  public void metAJour() {
  	super.metAJour();
  	// Bug corrig version 4.0 : il faut mettre  jour la liste d'lments anctres car metAJour est appel quand
  	// on modifie un obejt dont dpend cet objet
  	// if (nVersion == 1) etablitListeElementsAncetres();
  	etablitListeElementsAncetres();
  }
  
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double valeur, valeurMinimale, valeurMaximale, valeurInitiale, pas;
    
    if (! variableGeneratrice.existe) {
      existe = false;
      return;
    }
    
    valeurMinimale = variableGeneratrice.valeurMini;
    valeurMaximale = variableGeneratrice.valeurMaxi;
    valeurInitiale = variableGeneratrice.valeurActuelle;
    pas = (valeurMaximale-valeurMinimale)/(infoLieu.nombreDePoints-1);
    valeur = valeurMinimale - pas;
    boolean ligneEnCours = false;
    int indiceLigneEnCours = -1;
    int nb = infoLieu.nombreDePoints - 1;
    for (int i = 0; i < infoLieu.nombreDePoints; i++) {
    	// Optimisation version 4.0 pour viter les erreurs d'arrondi
    	valeur = (i == 0) ? valeurMinimale : ((i == nb) ? valeurMaximale : valeur + pas);
      variableGeneratrice.donneValeur(valeur);
      listeElementsAncetres.positionne(infoRandom, dimfen);
      if (!pointATracer.existe) {
        if (ligneEnCours) {
          infoLignes[indiceLigneEnCours].nombrePoints =
            i - infoLignes[indiceLigneEnCours].indicePremierPoint;
          ligneEnCours = false;
        }
      } else {
        if (!(Kernel.testToile(pointATracer.x,pointATracer.y))) {
          if (ligneEnCours) {
            infoLignes[indiceLigneEnCours].nombrePoints =
              i - infoLignes[indiceLigneEnCours].indicePremierPoint;
            ligneEnCours = false;
          }
        }
        else {
          if (ligneEnCours) {
            if (infoLieu.gestionDiscontinuite) {
              if ((Math.abs(pointATracer.x - abscissesCoordonneesPoints[i-1]) > dimfen.longueurAbscisses)
              || (Math.abs(pointATracer.y - ordonneesCoordonneesPoints[i-1]) > dimfen.longueurOrdonnees)) {
                infoLignes[indiceLigneEnCours].nombrePoints =
                  i - infoLignes[indiceLigneEnCours].indicePremierPoint;
                ligneEnCours = false;
              }
              else {
                abscissesCoordonneesPoints[i] = (int) Math.round(pointATracer.x);
                ordonneesCoordonneesPoints[i] = (int) Math.round(pointATracer.y);
                infoLignes[indiceLigneEnCours].nombrePoints++;
              }
            }
            else {
              abscissesCoordonneesPoints[i] = (int) Math.round(pointATracer.x);
              ordonneesCoordonneesPoints[i] = (int) Math.round(pointATracer.y);
              infoLignes[indiceLigneEnCours].nombrePoints++;
            }
          }
          else {
            abscissesCoordonneesPoints[i] = (int) Math.round(pointATracer.x);
            ordonneesCoordonneesPoints[i] = (int) Math.round(pointATracer.y);
            indiceLigneEnCours++;
            ligneEnCours = true;
            infoLignes[indiceLigneEnCours].indicePremierPoint = i;
            infoLignes[indiceLigneEnCours].nombrePoints = 1;
          }
        }
      }
    }
    // On redonne  la variable gnratrice sa valeur initiale
    variableGeneratrice.donneValeur(valeurInitiale);
    listeElementsAncetres.positionne(infoRandom, dimfen);
    nombreLignes = indiceLigneEnCours + 1;
    // Si le lieu n'est form d'aucune ligne ou s'il est form
    // de lignes avec un seul point, il n'existe pas
    if (nombreLignes == 0) {
      existe = false;
      return;
    }
    boolean auMoinsUneLigne = false;
    indicePremiereLigne = -1;
    nombreLignesDePlusieursPoints = 0;
    for (int j = 0; j <= indiceLigneEnCours; j++) {
      boolean ligneTrouvee = (infoLignes[j].nombrePoints >= 2);
      if (ligneTrouvee) {
        nombreLignesDePlusieursPoints++;
        if (!auMoinsUneLigne) indicePremiereLigne = j;
        auMoinsUneLigne = true;
      }
    }
    if (!auMoinsUneLigne) {
      existe = false;
      return;
    }
    // Si le lieu est dclar comme ferm, il n'existera que s'il
    // n'est form que d'une seule ligne avec au moins deux points
    
    if (infoLieu.fermeOuNon) {
    	// Modifi version 3.9. On accepte de fermer les lieux forms de plusieurs lignes
    	/*
      if (nombreLignes != 1) {
        existe = false;
        return;
      }
      */
      // On rajoute un dernier point gal de mme coordonnes
      // que le premier et on augmente le nombre de points de la ligne
    	// Modifi version 3.9. On ferme mme s'il y a plusieurs lignes
      //int indicePointRajoute = infoLignes[indicePremiereLigne].indicePremierPoint+
      //  infoLignes[indicePremiereLigne].nombrePoints;
      int indicePointRajoute = infoLignes[indiceLigneEnCours].indicePremierPoint+
       infoLignes[indiceLigneEnCours].nombrePoints;
      abscissesCoordonneesPoints[indicePointRajoute] =
        abscissesCoordonneesPoints[infoLignes[indicePremiereLigne].indicePremierPoint];
      ordonneesCoordonneesPoints[indicePointRajoute] =
        ordonneesCoordonneesPoints[infoLignes[indicePremiereLigne].indicePremierPoint];
      // infoLignes[indicePremiereLigne].nombrePoints++;
      infoLignes[indiceLigneEnCours].nombrePoints++;
    }
    existe = true;
  }
  
  public boolean confonduAvec(CElementBase p) {
    CLieuParVariable pt;
    if (p.getClass() == this.getClass()) {
      pt = (CLieuParVariable) p;
      return (pointATracer == pt.pointATracer) && (variableGeneratrice == pt.variableGeneratrice);
    } else return false;
  }
    
  public void etablitListeElementsAncetres() {
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (listeProprietaire.getClass() != CPrototype.class) {
	    listeElementsAncetres.retireTout();
	    int indicePointATracer = listeProprietaire.indexOf(pointATracer);
	    int indiceVariableGeneratrice = listeProprietaire.indexOf(variableGeneratrice);
	    listeElementsAncetres.add(variableGeneratrice);
      for (int i = indiceVariableGeneratrice + 1; i < indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
      listeElementsAncetres.add(pointATracer);
    }	
  }
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list,nVersion >= 2);  // modification version 2.5 gestion discontinuit des lieux
    int ind1 = inps.readCInt();
    variableGeneratrice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      int indicePointATracer = list.indexOf(pointATracer);
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        // Modification apporte pour optimisation lors de la version 1.9.5
        //if (pointATracer.dependDe(ptelb))	listeElementsAncetres.add(ptelb);
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
    }
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    variableGeneratrice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Modifi version 3.4.2
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    /*
    if (list.getClass() != CPrototype.class) {
      int indicePointATracer = list.indexOf(pointATracer);
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        // Modification apporte pour optimisation lors de la version 1.9.5
        //if (pointATracer.dependDe(ptelb))	listeElementsAncetres.add(ptelb);
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
    }
    */
    if (nVersion == 1) etablitListeElementsAncetres();
    else listeElementsAncetres.read(inps, list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(variableGeneratrice);
    oups.writeInt(ind1);
    listeElementsAncetres.write(oups, listeProprietaire);
  }
}
