package objets;

import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;
import types.ErreurCalculException;
import types.NatureCalcul;

public class CMaxFonc extends CCalculAncetre {
  public CFonc fonctionAssociee;
  // Les bornes de l'intervalle de recherche
  public CValeurAssocieeAVariable a;
  public CValeurAssocieeAVariable b;
  // L'incertitude recherche
  public CValeurAssocieeAVariable incertitude;
  // La valeur de la solution de l'quation
  protected double valeur;
  
  /** Creates new CSolutionEquation */
  public CMaxFonc() {
  }
  
  // Constructeur pour clonage
  public CMaxFonc(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomInit, CFonc fonctionAssocieeInit,
    CValeurAssocieeAVariable aInit, CValeurAssocieeAVariable bInit, CValeurAssocieeAVariable incertitudeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    fonctionAssociee = fonctionAssocieeInit;
    a = aInit;
    b = bInit;
    incertitude = incertitudeInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable aClone = (CValeurAssocieeAVariable) a.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable bClone = (CValeurAssocieeAVariable) b.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable incertitudeClone = (CValeurAssocieeAVariable) incertitude.getClone(listeSource, listeCible);
    CElementBase ptelb = new CMaxFonc(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFonc)(listeCible.get(ind1, CFonc.class)), aClone,
      bClone, incertitudeClone);
    return ptelb;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMaximumFonction;
  }
  
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	a.initialisePourDependance();
  	b.initialisePourDependance();
  	incertitude.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || a.dependDe(p) || b.dependDe(p)
    || incertitude.dependDe(p) || fonctionAssociee.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || a.dependDePourBoucle(p) || b.dependDePourBoucle(p)
    || incertitude.dependDePourBoucle(p) || fonctionAssociee.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double x1,x2,xa,xb,y1,y2,e,h;
    a.positionne(infoRandom, dimfen);
    b.positionne(infoRandom, dimfen);
    incertitude.positionne(infoRandom, dimfen);
    existe = fonctionAssociee.existe && a.existe() && b.existe() && incertitude.existe() ;
    if (!existe) return;
    xa = a.rendValeur(infoRandom);
    xb = b.rendValeur(infoRandom);
    e = incertitude.rendValeur(infoRandom);
    if (xa > xb) {
      double temp = xa;
      xa = xb;
      xb = temp;
    }

    // Contrairement  la mthode de dichotomie, la longueur de l'intervalle est ici multiplie  chaque fois par 2/3
    double nBoucles = Math.floor(Math.log(Math.abs(xb - xa)/e)/Math.log(1.5)) + 1;
    if (nBoucles > 1000) {
      existe = false;
      return;
    }
    h = (xb - xa)/3;
    x1 = xa + h;
    x2 = xb - h;
    for (int i = 1; i <= (int) nBoucles; i ++) {
    	h = (xb - xa)/3;
      x1 = xa + h;
      x2 = xb - h;
      try {
        y1 = fonctionAssociee.rendValeurFonction(infoRandom,x1);
        y2 = fonctionAssociee.rendValeurFonction(infoRandom,x2);
      } catch (ErreurCalculException err) {
        existe = false;
        return;
      }
      if (y1 > y2) xb = x2;
      else xa = x1;
    }
    existe = true;
    valeur = (x1 + x2)/2;
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return valeur;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = description() + " " + nom() + " " + getString("chinfo94") + " " + fonctionAssociee.nom() + "\n" +
    getString("chinfo100") + " " + a.chaineInfo() + " " + getString("chinfo101") + " " + b.chaineInfo() + "\n" +
    getString("chinfo102") + " " + incertitude.chaineInfo();
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + glob.Kernel.chaineNombre(valeur, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
  
  protected String description() {
  	return getString("chinfo176");
  }
 
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    a = new CValeurAssocieeAVariable();
    a.chargeDepuisFlux(inps, list);
    b = new CValeurAssocieeAVariable();
    b.chargeDepuisFlux(inps, list);
    incertitude = new CValeurAssocieeAVariable();
    incertitude.chargeDepuisFlux(inps, list);
  }

   @Override
   public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    a = new CValeurAssocieeAVariable();
    a.read(inps, list);
    b = new CValeurAssocieeAVariable();
    b.read(inps, list);
    incertitude = new CValeurAssocieeAVariable();
    incertitude.read(inps, list);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
    a.write(oups, list);
    b.write(oups, list);
    incertitude.write(oups, list);
  }
  
  @Override
	public boolean estConstant() {
		return a.estConstant() && b.estConstant() && incertitude.estConstant() && fonctionAssociee.estConstant();
	}  
  
}
