/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.Complexe;
import types.ErreurCalculException;
import types.OperateurFonction;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

public class CPuissance extends CCalculBase {
  CCalculBase operande;
  CCalculBase exposant;
  // Attribut rajout pour optimisation
  Complexe z1 = new Complexe();
  Complexe ope1 = new Complexe();
  Complexe exp1 = new Complexe();

  /** Creates new CPuissance */
  public CPuissance() {
  }
  
  // Constructeur pour clonage
  public CPuissance(CListeObjets liste, CCalculBase operandeInit, CCalculBase exposantInit) {
    super(liste);
    operande = operandeInit;
    exposant = exposantInit;
  }

  @Override
  protected long nature() {
  	return natPuissance;
  }  
  
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande = operande.getClone(listeSource, listeCible); 
    CCalculBase cloneExposant = exposant.getClone(listeSource, listeCible); 
    //  revoir car la fonction a dj t cre
    CCalculBase ptcalc = new CPuissance(listeCible, cloneOperande, cloneExposant);
    return ptcalc;
  }
  
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	operande.initialisePourDependance();
  	exposant.initialisePourDependance();
  }

  // Modifi version 4.8
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || operande.dependDe(p) || exposant.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	return operande.dependDePourReclassement(p) || exposant.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return operande.dependDePourBoucle(p) || exposant.dependDePourBoucle(p);
  }

  @Override
  public boolean existe(boolean infoRandom) {
    return operande.existe(infoRandom) && exposant.existe(infoRandom);
  }

  // La procdure suivante a t optimise pour la version 2.0
  public double resultatBase(double x, double n) throws ErreurCalculException {
    if((x == 0) && (n != 0))
    {
      if (n>0) return 0;
      else 
      {
        throw new ErreurCalculException();
      }
    }
    if (n == 0)
    {
      if (x == 0)
      {
        throw new ErreurCalculException();
      }
      else return 1;
    }
    else
    {
      // Optimisation pour la version 2.0
      // Pour un exposant entier compris entre 1 et 255
      // on ne fait que des produits
      if ((n == Math.floor(n)) && (n > 0) && (n < 256)) return CCalculBase.PuisExpEnt(x, (int)n);
      else
      {
        if (x < 0)
        {
          if (n == Math.floor(n))
          {
          	// Modifi version 4.8 pour viter test d'galit en virgule flottante
            // if (n == 2*Math.floor(n/2))
          	if (Kernel.zero13(n - 2*Math.floor(n/2)))
            return Math.exp(n*Math.log(Math.abs(x)));
            else return -Math.exp(n*Math.log(Math.abs(x)));
          }
          else
          {
            throw new ErreurCalculException();
          }
        }
        else return Math.exp(n*Math.log(x));
      }
    }
  }
  
  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
    double x1 = operande.resultat(infoRandom);
    double n1 = exposant.resultat(infoRandom);
    return resultatBase(x1, n1);   
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
    double x1 = operande.resultatFonction(infoRandom, valeurParametre);
    double n1 = exposant.resultatFonction(infoRandom, valeurParametre);
    return resultatBase(x1, n1);
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
    double x1 = operande.resultatFonction(infoRandom, valeurParametre);
    double n1 = exposant.resultatFonction(infoRandom, valeurParametre);
    return resultatBase(x1, n1);
  }
  
  
  void resultatComplexeBase(boolean infoRandom, Complexe ope, Complexe exp, Complexe zRes)  throws ErreurCalculException {
    boolean openull = ope.moduleCarre() == 0;
    boolean expnull = exp.moduleCarre() == 0;
    if (expnull)
    {
      if (openull)
      {
        throw new ErreurCalculException();
      }
      else
      {
        zRes.x = 1;
        zRes.y = 0;
        return;
      }
    }
    else
    {
      if (openull)
      {
        zRes.x = 0;
        zRes.y = 0;
        // Si l'oprande est nul, on regarde si l'exposant est rel ou pas.
        // S'il ne l'est pas, le rsultat n'existe pas
        if (exp.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          if (exp.x > 0)
          {
            return;
          }
          else
          {
            throw new ErreurCalculException();
          }
        }
      }
      else // Si l'oprande n'est pas nul ni l'exposant
      {
        // On regarde si l'exposant est rel et strictemnt infrieur  256.
        // Si oui on utilise un algorithme n'utilisant que des produits
        if ((exp.y == 0) && (exp.x == Math.floor(exp.x))) {
          if ((exp.x>0) && (exp.x < 256))
          {
            int expent = (int)exp.x;
            ope.puisCompExpEnt(expent, zRes);
          }
          else
          {
            if ((exp.x<0) && (exp.x > -256))
            {
              int expent = (int)exp.x;
              ope.puisCompExpEnt(-expent, z1);
              z1.InverseComp(zRes);
            }
            else ope.puisComp(exp, zRes);
          }
        }
        else
        {
          ope.puisComp(exp, zRes);
        }
      }
    }    
  }
  
  @Override
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
    operande.resultatComplexe(infoRandom, ope1);
    exposant.resultatComplexe(infoRandom, exp1);
    resultatComplexeBase(infoRandom, ope1, exp1, zRes);
  }
  
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
                            throws ErreurCalculException {
    operande.resultatFonctionComplexe(infoRandom, valeurParametre, ope1);
    exposant.resultatFonctionComplexe(infoRandom, valeurParametre, exp1);
    resultatComplexeBase(infoRandom, ope1, exp1, zRes);
  }

  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
                            throws ErreurCalculException {
    operande.resultatFonctionComplexe(infoRandom, valeurParametre, ope1);
    exposant.resultatFonctionComplexe(infoRandom, valeurParametre, exp1);
    resultatComplexeBase(infoRandom, ope1, exp1, zRes);
  }
  
  @Override
  public boolean  dependDeVariable(int ind) {
    return operande.dependDeVariable(ind) || exposant.dependDeVariable(ind);
  }
  
  @Override
  public CCalculBase getCopie() {
  	return new CPuissance(listeProprietaire, operande.getCopie(), exposant.getCopie());
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	// varForBug corrig version 4.5.3.2
		// return operande.chaineCalcul() + "^" + exposant.chaineCalcul(varFor);
  	String chexp = exposant.chaineCalcul(varFor);
  	long natexp = exposant.nature();
  	boolean expestpuis = (natexp == natPuissance);
  	if (natexp == natFonction) {
  		CFonction fonc = (CFonction) exposant;
  		expestpuis = expestpuis || (fonc.operateurFonction == OperateurFonction.Exp) || (fonc.operateurFonction == OperateurFonction.Expo);
  	}
  	if (expestpuis) chexp = parouv + chexp + parfer;
  	long natop = operande.nature();
  	boolean opestpuis = (natop == natPuissance);
  	if (opestpuis) return parouv +  operande.chaineCalcul(varFor) + parfer + "^" + chexp;
  	else return operande.chaineCalcul(varFor) + "^" + chexp;
	}
 
  @Override
	public String chaineLatex(String[] varFor) {
  	// Bug corrig version 4.5.3.2
		// return operande.chaineLatex(varFor) + "^{" + exposant.chaineLatex(varFor) + "}";
  	String chexp = exposant.chaineLatexSansPar(varFor); // Parenthses inutiles pour le LaTeX
  	// Modifi version 4.6
  	/*
  	long natop = operande.nature();
  	boolean estpuis = (natop == natPuissance);
  	if (natop == natFonction) {
  		CFonction fonc = (CFonction) operande;
  		estpuis = estpuis || (fonc.operateurFonction == OperateurFonction.Exp) || (fonc.operateurFonction == OperateurFonction.Expo);
  	}
  	if (estpuis) return parouvLatex +  operande.chaineLatex(varFor) + parferLatex + "^{" + chexp + "}";
  	else return operande.chaineLatex(varFor) + "^{" + chexp + "}";
  	*/
  	long natop = operande.nature();
  	// Bug corrig version 4.7.2.1
  	if (((natop & (natPuissance | natOperation | natMoinsUnaire)) != 0) || ((natop & (natFonction | natFonction2Var | natFonction3Var)) != 0))
  		return parouvLatex +  operande.chaineLatexSansPar(varFor) + parferLatex + "^{" + chexp + "}";
  		else return operande.chaineLatexSansPar(varFor) + "^{" + chexp + "}";
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return 
  	operande.estDefiniParObjetDans(listeOb)
		&&
		exposant.estDefiniParObjetDans(listeOb);
  	
	}	   
	
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    operande = (CCalculBase) (inps.readCObject(list));
    exposant = (CCalculBase) (inps.readCObject(list));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operande = (CCalculBase) (inps.readObject(list));
    exposant = (CCalculBase) (inps.readObject(list));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeObject(operande);
    oups.writeObject(exposant);
  }
  
  @Override
	public boolean estConstant() {
		return operande.estConstant() && exposant.estConstant();
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return operande.deriveePossible(indiceVariable) && exposant.deriveePossible(indiceVariable);
  }
  
  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CPuissance(listeProprietaire, operande.calculAvecValeursRemplacees(), exposant.calculAvecValeursRemplacees()) ;
  }
  
}
