/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 *
 * @author Yves
 */
/* Classe ajoute pour la version 2.5 */
public class CSomme extends CCalculAncetre {
  private final static int NombreMaxiTermesSomme = 5000;
  // La liste des lments de la figure dont dpendra la somme indice
  CSousListeObjets listeElementsAncetres;
  
  public CVariableBornee indice;
  public CValeurDynamique valeurASommer;
  // Les bornes de l'intervalle de sommation
  public CValeurAssocieeAVariable indiceMin;
  public CValeurAssocieeAVariable indiceMax;
  double valeurSomme;
  /** Creates a new instance of CSomme */
  public CSomme() {
  }
  
  // Constructeur pour clonage
  public CSomme(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, CVariableBornee indiceInit, CValeurDynamique valeurASommerInit,
    CValeurAssocieeAVariable indiceMinInit, CValeurAssocieeAVariable indiceMaxInit) {
    super(liste, impProtoInit, estElementFinalInit,  nomCalculInit);
    indice = indiceInit;
    valeurASommer = valeurASommerInit;
    indiceMin = indiceMinInit;
    indiceMax = indiceMaxInit;
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Cette liste sera finie d'tre valide dans setClone()
    listeElementsAncetres = new CSousListeObjets();
    // Ligne suivante ajoute version 3.4.2
    // etablitListeElementsAncetres();  Supprim version 4.9.4 appel dans metAJour()
  }
  
  /**
   * A partir de la version 3.4.2, changement de version (version 1 -> 2) car on enregistre dans le flux la CSousListeObjetstres
   * contenant les pointeurs sur les lments anctres
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(indice);
    int ind2 = listeSource.indexOf(valeurASommer);
    int ind3 = listeSource.indexOf(impProto);    
    CValeurAssocieeAVariable indiceMinClone = (CValeurAssocieeAVariable) indiceMin.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable indiceMaxClone = (CValeurAssocieeAVariable) indiceMax.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSomme(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CVariableBornee)(listeCible.get(ind1, CVariableBornee.class)),
      (CValeurDynamique)listeCible.get(ind2, CValeurDynamique.class), indiceMinClone, indiceMaxClone);
    if (listeCible.getClass() != CPrototype.class) {    
	    ((CSomme)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire,
	      listeCible);
    }
    return ptelb;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NSommeIndicee;
  }
    
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	indiceMin.initialisePourDependance();
  	indiceMax.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || indice.dependDe(p) || valeurASommer.dependDe(p)
    || indiceMin.dependDe(p) || indiceMax.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || indice.dependDePourBoucle(p) || valeurASommer.dependDePourBoucle(p)
    || indiceMin.dependDePourBoucle(p) || indiceMax.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	indiceMin.positionne(infoRandom, dimfen);
  	indiceMax.positionne(infoRandom, dimfen);
    existe = indiceMin.existe() && indiceMax.existe();
    if (!existe) return;
    // On arrondit les indices de dpart et de fin  l'entier le plus proche
    double indmin = Math.floor(indiceMin.rendValeur(infoRandom) + 0.5);
    double indmax = Math.floor(indiceMax.rendValeur(infoRandom) + 0.5);
    existe = (indmin <= indmax) && ((indmax - indmin) < NombreMaxiTermesSomme) ;
    if (!existe) return;
    double sauveIndice = indice.rendValeur(infoRandom);
    valeurSomme = 0;
    //  revoir : faut-il que la variable de boucle soit double ?
    for (double i = indmin; i <= indmax; i++)
    {
      indice.valeurActuelle = i;
      // listeElementsAncetres.positionne(infoRandom, dimfen); // Moifi version 4.9.3
      listeElementsAncetres.positionneFull(infoRandom, dimfen);
      existe = existe && valeurASommer.existe;
      valeurSomme = valeurSomme + valeurASommer.rendValeur(infoRandom);
    }
    indice.donneValeur(sauveIndice);
    listeElementsAncetres.metAJour(); // Rajout version 4.9.6
    listeElementsAncetres.positionne(infoRandom, dimfen);    
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return valeurSomme;
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
  public void etablitListeElementsAncetres() {
  	listeElementsAncetres.retireTout();
    int indiceValeurASommer = listeProprietaire.indexOf(valeurASommer);
    for (int i = 0; i <= indiceValeurASommer; i++) {
      CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
      if ((ptelb != indice) && (valeurASommer.dependDe(ptelb)) && (ptelb.dependDe(indice)))
        listeElementsAncetres.add(ptelb);
    }  	
  }
  
  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   */
  @Override
  public void metAJour() {
  	super.metAJour();
  	// Bug corrig version 4.0 : il faut mettre  jour la liste d'lments anctres car metAJour est appel quand
  	// on modifie un obejt dont dpend cet objet
  	// if (nVersion == 1) etablitListeElementsAncetres();
  	etablitListeElementsAncetres();
  	// Supprim version 3.4.2 car la liste d'lments anctres est enregistre dans le flux
  	// etablitListeElementsAncetres();
  }
  
   
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    listeElementsAncetres = new CSousListeObjets();
    int ind1 = inps.readCInt();
    indice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    int ind2 = inps.readCInt();
    valeurASommer = (CValeurDynamique) (list.get(ind2,CValeurDynamique.class));
    indiceMin = new CValeurAssocieeAVariable();
    indiceMin.chargeDepuisFlux(inps, list);
    indiceMax = new CValeurAssocieeAVariable();
    indiceMax.chargeDepuisFlux(inps, list);
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
    	etablitListeElementsAncetres();
    }    
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    listeElementsAncetres = new CSousListeObjets();
    int ind1 = inps.readInt();
    indice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    int ind2 = inps.readInt();
    valeurASommer = (CValeurDynamique) (list.get(ind2,CValeurDynamique.class));
    indiceMin = new CValeurAssocieeAVariable();
    indiceMin.read(inps, list);
    indiceMax = new CValeurAssocieeAVariable();
    indiceMax.read(inps, list);
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (nVersion == 1) etablitListeElementsAncetres();
    else listeElementsAncetres.read(inps, list);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(indice);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(valeurASommer);
    oups.writeInt(ind2);
    indiceMin.write(oups, list);
    indiceMax.write(oups, list);
    listeElementsAncetres.write(oups, listeProprietaire);
  }
}
