/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
// Classe anctre de toutes les classes de surface possdant un bord
import java.io.*;
import java.awt.Color;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public abstract class CSurface extends CSurfaceAncetre {
	CElementGraphique bord;	// Le bord qui sera du type cercle,

  /** Creates new CSurface */
  public CSurface() {
  }
  
  // Constructeur pour clonage
  public CSurface(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleRemplissageInit, CElementGraphique bordInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit);
    bord = bordInit;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
  	boolean res = super.confonduAvec(p);
  	if (!res) return false;
  	CSurface pt = (CSurface) p;
  	res = res && (bord == pt.bord);
  	return res;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || bord.dependDe(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || bord.dependDePourBoucle(p));
  }

  @Override
  public boolean dependDePourCapture(CElementBase p) {
    return ((p == this) || bord.dependDePourCapture(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = bord.existe;
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	if (!existe || (masquage && masque)) return -1;
  	return bord.distancePointPourSurface(xp, yp, masquage);
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    bord = (CElementGraphique) (list.get(ind1,CElementGraphique.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    bord = (CElementGraphique) (list.get(ind1,CElementGraphique.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(bord);
    oups.writeInt(ind1);
  } 
}
