/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import java.awt.Color;
import java.io.IOException;

import types.NatureObjet;
import types.StyleRemplissage;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


public abstract class CSurfaceAncetre extends CElementGraphique {
  
	public int styleRemplissage; // Nom chang parrapport au C++ (Quadrillage) Etait un int en C++
	
	/** Creates new CSurfaceAncetre */

  public CSurfaceAncetre() {
  }

  // Constructeur pour clonage
  public CSurfaceAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleRemplissageInit) {
   super(liste, impProtoInit, estElementFinalInit, couleurInit, false, 0, 0, masqueInit, "", 0);
   styleRemplissage = styleRemplissageInit;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NSurface;
  }
  
  /**
   * Fonction appele quand on change le nombre de points du lieu qui est le bord de la surface
   * A redfinir pour les descendants atant un lieu pour bord
   */
  public void metAJourTableaux() {
  }
 
  /**
   * Fonction renvoyant true lorsque la surface a pour bord lieu
   * Utilis quand on change le nombre de points d'un lieu. Il faut alors recrer les tableaux
   * utilis par les surfaces ayant lieu pour bord
   * @param lieu
   * @return
   */
  public boolean aPourBord(CLieuDeBase lieu) {
  	return false;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CSurfaceAncetre pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSurfaceAncetre) p;
      return (styleRemplissage == pt.styleRemplissage);
    }
    else return false;
  }  
  
  /**
   * Ajout pour version 3.0. Seules les surfaces CSurfaceDisque, CSurfaceSecteurCirculaire et CSurfacePolygone
   * peuvent gnrer des leiux d'objets
   * Donc  redfinir pour ces trois objets
   * @return
   */
  public boolean peutGenererLieuObjet() {
  	return false;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    styleRemplissage = inps.readCInt();
    // Version 4.9.2 : On remplace les remplissages pleins par des remplissages transparents
    // Et les quadrillages obliques ont un se,s invers par rapport  la version C++
    switch (styleRemplissage) {
		case StyleRemplissage.QuadrillagePlein :
			styleRemplissage = StyleRemplissage.QuadrillagePleinTransp;
			break;
		case StyleRemplissage.Quadrillage45Bas :
			styleRemplissage = StyleRemplissage.Quadrillage45Haut;
			break;
		case StyleRemplissage.Quadrillage45Haut :
			styleRemplissage = StyleRemplissage.Quadrillage45Bas;
			break;
		}
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    styleRemplissage = inps.readInt();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(styleRemplissage);
  }    
}