/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Surface dlimite par un polygone
import java.awt.*;
import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import types.*;

/** Classe reprsentant une surface dlimite par un arc de cercle
 * Modifie version 3.0 pour pouvoir tre un objet gnrant un lieu d'objets
 * @author Yves
 *
 */
public class CSurfacePolygone extends CSurface {
	private CPolygone polygone; // Ajout version 3.0

  /** Creates new CSurfacePolygone */
  public CSurfacePolygone() {
  }
  
  // Constructeur pour clonage
  public CSurfacePolygone(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleRemplissageInit, CPolygone bordInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit, bordInit);
    polygone = new CPolygone(liste, null, false, Color.black, false, StyleTrait.traitFin,
    		((CPolygone)bordInit).collectionPoints);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(bord);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSurfacePolygone(listeCible, (CImplementationProto)listeCible.get(ind2, CImplementationProto.class),
      estElementFinal, couleur, masque, styleRemplissage, (CPolygone)listeCible.get(ind1, CPolygone.class));
    return ptelb;    
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	if (!existe || (masquage && masque)) return -1;
  	return polygone.distancePointPourSurface(xp, yp, masquage);
  }  
  
  // Rajout version 3.0 pour pouvoir tre un objet gnrant un lieu d'objets
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  	polygone.setClone((CPolygone) bord);
  }  
  
  // Ajout version 3.0
  @Override
	public void setClone(CElementBase ptel) {
		super.setClone(ptel);
		polygone.setClone(((CSurfacePolygone) ptel).polygone);
  }
  
  @Override  
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {

    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    pDC.fillPolygon(polygone.abscissesCoordonneesPoints, polygone.ordonneesCoordonneesPoints, 
  	        polygone.collectionPoints.size());
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(polygone.abscissesCoordonneesPoints, polygone.ordonneesCoordonneesPoints, 
    	      polygone.collectionPoints.size());
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(polygone.abscissesCoordonneesPoints, polygone.ordonneesCoordonneesPoints, 
    				polygone.collectionPoints.size());
      }
      else {
	    	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire.basseDefinition, couleurFond);
	    	pDC.setPaint(tp);
	    	pDC.fillPolygon(polygone.abscissesCoordonneesPoints, polygone.ordonneesCoordonneesPoints, 
	    			polygone.collectionPoints.size());
      }
    }
  }

  @Override
  public boolean peutGenererLieuObjet() {
  	return true;
  }
  
  // Ajout version 3.2.0 pour compatibilit version C++
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    polygone = new CPolygone(list, null, false, Color.black, false, StyleTrait.traitFin,
    		((CPolygone)bord).collectionPoints);    
  }
  
  // Ajout version 3.0 pour initialiser arc
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    polygone = new CPolygone(list, null, false, Color.black, false, StyleTrait.traitFin,
    		((CPolygone)bord).collectionPoints);
  }
  
}
