/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

// Classe de calcul servant  savoir si une autre valeur existe
// ou non. Vaut 1 si la valeur existe et 0 sinon.
import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CTestExistence extends CCalculAncetre {
	public CValeurDynamique valeurAssociee;

  /** Creates new CTestExistence */
  public CTestExistence() {
  }

  // Constructeur pour clonage
  public CTestExistence(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CValeurDynamique valeurAssocieeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    valeurAssociee = valeurAssocieeInit;
  }
   
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(valeurAssociee);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CTestExistence(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CValeurDynamique)(listeCible.get(ind1, CValeurDynamique.class)));
    return ptelb;
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NTestExistence;
  }
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || valeurAssociee.dependDe(p));
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || valeurAssociee.dependDePourBoucle(p);
  }

  @Override
  public double rendValeur(boolean infoRandom) {
    if (valeurAssociee.existe) return 1;
    else return 0;
  }
  
  public boolean confonduAvec(CElementBase p) {
    CTestExistence pt;
    if (p.getClass() == this.getClass()) {
      pt = (CTestExistence) p;
      return valeurAssociee == pt.valeurAssociee;
    }
    else return false;
  }    
   
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    valeurAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  }
  
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    valeurAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(valeurAssociee);
    oups.writeInt(ind1);
  }          
}
