/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import java.awt.geom.Point2D;
import java.io.IOException;
import types.DimensionFenetre;
import types.VecteurDirecteur;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CTransParVect extends CTransformation {
	public CVecteur vecteurTrans;

  /** Creates new CTranslation */
  public CTransParVect() {
  }
  
  @Override
	public int natureTransformation() {
		return translation;
	}
  
  
  /** Constructeur pour clonage */
  public CTransParVect(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CVecteur vecteurTransInit) {
    super(liste, impProtoInit, estElementFinalInit);
    vecteurTrans = vecteurTransInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(vecteurTrans);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CTransParVect(listeCible, (CImplementationProto)listeCible.get(ind2, CImplementationProto.class),
      estElementFinal, (CVecteur)listeCible.get(ind1, CVecteur.class));
    return ptelb;    
  }
  
  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    liste.add(vecteurTrans);
  }
  
  
  String designationTransf() {
    return getString("chinfo19");
  } 
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || vecteurTrans.dependDe(p));
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || vecteurTrans.dependDePourBoucle(p);
  }


  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = vecteurTrans.existe;
  }

  public boolean confonduAvec(CElementBase p) {
    CTransParVect pt;
    if (p.getClass() == this.getClass()) {
      pt = (CTransParVect) p;
      return (vecteurTrans == pt.vecteurTrans);
    } else return false;
  }
  
  public double transformeLongueur(double L) {
    return L;
  }

  public void transformePoint(double x, double y, Point2D.Double pointr) {
    pointr.x = x + vecteurTrans.x2 - vecteurTrans.x1;
    pointr.y = y + vecteurTrans.y2 - vecteurTrans.y1;
  }

  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
    u.setCopy(ur);
  }
  
  /** Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    vecteurTrans = (CVecteur) (list.get(ind1,CVecteur.class));
  }
  
  /** Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(vecteurTrans);
    oups.writeInt(ind1);
  }
}
