/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import Applet.AppletMathGraph32;
import glob.*;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import objets.CDroite;
import objets.CElementGraphique;
import objets.CListeObjets;

import types.InfoProx;
import types.NatureObjet;

/**
 *
 * @author Yves
 */
public class OutilPointagePourNommer extends OutilPointageCre {
  private CElementGraphique eltAyantNomCapturable;
  int x, y ; // Coordonnes du pointeur souris lors du dernier mouseMove
  
  /** Creates a new instance of OutilPointageObjetClignotant */
  public OutilPointagePourNommer(AppletMathGraph32 appletMereInit) {
    super(appletMereInit);
  }
  
  @Override
  public void reset() {
    eltAyantNomCapturable = null;
  }
  
  /**
   * Renvoie la liste  utiliser.
   */
  @Override
  protected CListeObjets listeUtilisee() {
    return appletMere.listeConstruction;
  }

  @Override
  public void mouseMoved(java.awt.event.MouseEvent mouseEvent) {
    x = mouseEvent.getX();
    y = mouseEvent.getY();
    CListeObjets liste = listeUtilisee();
    InfoProx info = appletMere.infoProx;
    int nbObjetsProches = liste.procheDe(aDesigner, mouseEvent.getPoint(), info, 
      appletMere.listeExclusionDesignation);
    int nbTypesProches = info.nombreTypesProches;
    // true si une dsignation est dj affiche prs du pointeur
    if (nbObjetsProches > 0) {
        String ch = "";
      // Si lors d'un dialogue de choix d'objet proche ona dj choisi un objet
      // et si celui-ci est proche du pointeur souris, c'est celui-l qui sera slectionnn
      if (info.premierPointVoisin != null) {
        ch = getString("designationPoint");
      }
      else {
        // A revoir plus tard pour grer le cas o plusieurs objets sont proches
        if (nbTypesProches == 1) {
          for (int i =0; i < NatureObjet.nombreTypesDesignables; i++) {
            if (info.infoParType[i].nombreVoisins > 0) {
              ch = info.infoParType[i].premierVoisin.chaineDesignation();
              break;
            }
          }
        }
        else {
          ch = " prciser"; // A revoir
        }
      }
      appletMere.commentaireDesignation.donneChaine(ch);
      appletMere.commentaireDesignation.placeEn(x, y);
      appletMere.paneFigure.repaint();
      setActiveCursor(AppletMathGraph32.crossHairCursor);
    }
    else {
      // Si aucun point ou aucune droite n'est proche du pointeur souris
      // on regarde si un nom de point ou de droite est proche de celui-ci
      eltAyantNomCapturable = liste.premierObjetNomProcheDe((Graphics2D) appletMere.getGraphics(),
              aDesigner, mouseEvent.getPoint());
      // Si un nom est proche on transforme le curseur en curseur main
      if (eltAyantNomCapturable != null) setActiveCursor(AppletMathGraph32.handCursor);
      else setActiveCursor(AppletMathGraph32.defaultCursor);
      appletMere.videDesignation();
    }
  }  

  // Si on est en rain de vouloir dplacer le nom d'un point, on n'ouvre pas la
  // bote de dialogue d'entre de nom
  public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
    if (appletMere.getCursor() != AppletMathGraph32.handCursor) super.mousePressed(mouseEvent);
  }
  
  @Override
  public void mouseDragged(java.awt.event.MouseEvent mouseEvent) {
    double decxn, decyn;
    if ((x > appletMere.dimf.longueurAbscisses) || (y > appletMere.dimf.longueurOrdonnees)) {
      return;
    }
    int xn = mouseEvent.getX();
    int yn = mouseEvent.getY();
    if (eltAyantNomCapturable != null) {
      if ((eltAyantNomCapturable.getNature() & NatureObjet.NToutPoint) != 0) {
        decxn = xn - x + eltAyantNomCapturable.dec_x;
        decyn = yn - y + eltAyantNomCapturable.dec_y;
        if (decyn < -30) decyn = -30;
        else {
          if (decyn > 10) decyn = 10;
        }
        if (decxn < -25) decxn = -25;
        else {
          if (decxn > 15) decxn = 15;
        }
        eltAyantNomCapturable.decaleNom( decxn, decyn);
      }
      else {
        double nouvelleAbs;
        if ((eltAyantNomCapturable.getNature() & NatureObjet.NTouteDroite) != 0) {
          CDroite dte = (CDroite) eltAyantNomCapturable;
          Point2D.Double proj = new Point2D.Double();
          double x1 = eltAyantNomCapturable.x_nom + eltAyantNomCapturable.dec_x + xn - x;
          double y1 = eltAyantNomCapturable.y_nom + eltAyantNomCapturable.dec_y + yn - y;
          Kernel.projetteOrtho(x1, y1, dte.point_x, dte.point_y, dte.vecteur, proj);
          if (dte.xext1 != dte.xext2) nouvelleAbs = (proj.x-dte.xext1)/(dte.xext2-dte.xext1);
            else nouvelleAbs = (proj.y-dte.yext1)/(dte.yext2-dte.yext1);
          decxn = x1 - proj.x;
          decyn = y1 - proj.y;
          if (decyn < -30) decyn = -30;
          else {
            if (decyn > 10) decyn = 10;
          }
          if (decxn < -25) decxn = -25;
          else {
            if (decxn > 15) decxn = 15;
          }
          dte.decaleNom(decxn,decyn);
          dte.abscisseNom = nouvelleAbs;
        }      
      }
      x = mouseEvent.getX();
      y = mouseEvent.getY();
      eltAyantNomCapturable.positionne(appletMere.infRand, appletMere.dimf);
      appletMere.redessineSansCalcul();
    }
  }  
}
