/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;
/*

 *
 * Created on 17 juillet 2004, 21:28
 */

/**
 *
 * @author  Yves
 * @version 
 */
// Classe dfinissant les diffrents types de remplissage possibles pour les surfaces
// et fournissant des objets TexturePaint servant aux remplissages
// Modifi version 4.9.2 car BasseDefinition est toujours true
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.*;
public abstract class StyleRemplissage {

  
  public static final int QuadrillageVertical = 0;
  public static final int QuadrillageHorizontal = 1;
  public static final int Quadrillage45Haut = 2;
  public static final int Quadrillage45Bas = 3;
  public static final int QuadrillagePlein = 4;
  public static final int QuadrillagePleinTransp = 5;
  public static final int QuadrillageCroise = 6;
  
  public static final TexturePaint getTexturePaint(Color couleur, int styleRemplissage, boolean basseDefinition,
  		Color couleurFond) {
    BufferedImage bi = null;
    WritableRaster r = null;
    // On spcifie les 3 composantes pour le trait et le coefficient de transparence en dernier
    int[] couleurTrait = new int[4];
    couleurTrait[0] = couleur.getRed();
    couleurTrait[1] = couleur.getGreen();
    couleurTrait[2] = couleur.getBlue();
    couleurTrait[3] = 255; // Le trait remplace ce qu'il recouvre
    int[] coulFond = new int[4];
    coulFond[0] = couleurFond.getRed();
    coulFond[1] = couleurFond.getGreen();
    coulFond[2] = couleurFond.getBlue();
    Rectangle2D.Double rect = null;
    
    switch (styleRemplissage) {
      case QuadrillageVertical :
        bi = new BufferedImage(6, 6, BufferedImage.TYPE_4BYTE_ABGR);
        r = bi.getRaster();
        rect = new Rectangle2D.Double(0,0,6,6);	
        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures     
        for (int i = 0; i < 6; i++) {
          for (int j = 0; j < 6; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(0,1,couleurTrait);
        r.setPixel(0,2,couleurTrait);
        r.setPixel(0,3,couleurTrait);
        r.setPixel(0,4,couleurTrait);
        r.setPixel(0,5,couleurTrait);
        return new TexturePaint(bi, rect);
        
      case QuadrillageHorizontal :

        bi = new BufferedImage(6, 6, BufferedImage.TYPE_4BYTE_ABGR);
        r = bi.getRaster();
        rect = new Rectangle2D.Double(0,0,6,6);	
        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 6; i++) {
          for (int j = 0; j < 6; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(1,0,couleurTrait);
        r.setPixel(2,0,couleurTrait);
        r.setPixel(3,0,couleurTrait);
        r.setPixel(4,0,couleurTrait);
        r.setPixel(5,0,couleurTrait);
        return new TexturePaint(bi, rect);
        
      case Quadrillage45Bas :

        bi = new BufferedImage(8, 8, BufferedImage.TYPE_4BYTE_ABGR);
        r = bi.getRaster();
        rect = new Rectangle2D.Double(0,0,8,8);	
        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 8; i++) {
          for (int j = 0; j < 8; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(1,1,couleurTrait);
        r.setPixel(2,2,couleurTrait);
        r.setPixel(3,3,couleurTrait);
        r.setPixel(4,4,couleurTrait);
        r.setPixel(5,5,couleurTrait);
        r.setPixel(6,6,couleurTrait);
        r.setPixel(7,7,couleurTrait);
        return new TexturePaint(bi, rect);
        
      case Quadrillage45Haut :

        bi = new BufferedImage(8, 8, BufferedImage.TYPE_4BYTE_ABGR);
        r = bi.getRaster();
        rect = new Rectangle2D.Double(0,0,8,8);	
        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 8; i++) {
          for (int j = 0; j < 8; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(7,0,couleurTrait);
        r.setPixel(6,1,couleurTrait);
        r.setPixel(5,2,couleurTrait);
        r.setPixel(4,3,couleurTrait);
        r.setPixel(3,4,couleurTrait);
        r.setPixel(2,5,couleurTrait);
        r.setPixel(1,6,couleurTrait);
        r.setPixel(0,7,couleurTrait);
        return new TexturePaint(bi, rect);
        
      case QuadrillageCroise :

        bi = new BufferedImage(8, 8, BufferedImage.TYPE_4BYTE_ABGR);
        r = bi.getRaster();
        rect = new Rectangle2D.Double(0,0,8,8);	
        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
        for (int i = 0; i < 8; i++) {
          for (int j = 0; j < 8; j++) {
            r.setPixel(i, j, coulFond);
          }
        }
        r.setPixel(0,0,couleurTrait);
        r.setPixel(1,1,couleurTrait);
        r.setPixel(2,2,couleurTrait);
        r.setPixel(3,3,couleurTrait);
        r.setPixel(4,4,couleurTrait);
        r.setPixel(5,5,couleurTrait);
        r.setPixel(6,6,couleurTrait);
        r.setPixel(7,7,couleurTrait);
        r.setPixel(7,0,couleurTrait);
        r.setPixel(6,1,couleurTrait);
        r.setPixel(5,2,couleurTrait);
        r.setPixel(4,3,couleurTrait);
        r.setPixel(3,4,couleurTrait);
        r.setPixel(2,5,couleurTrait);
        r.setPixel(1,6,couleurTrait);
        r.setPixel(0,7,couleurTrait);
        return new TexturePaint(bi, rect);        
        
      default : return null; // Ne doit pas arriver
    }
  }
}